package de.historischekommission_muenchen.oxydition.brfedition.common.operations;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorDocument;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;

/**
 * Oxygen-Author-Operation, die den Dateinamen der aktuellen Datei ausliest
 * (ohne Pfad und Dateiendung) und diesen als xml:id-Attribut in den
 * TEI-Wurzelknoten und das <body>-Element einfügt.
 * 
 * @author Maximilian Schrott
 */
public class InsertLetterId extends OxyditionOperation {

	@Override
	public String getDescription() {
		return "Fügt den Dateinamen als @xml:id in den TEI-Wurzelknoten und (mit Präfix) in das <body>-Element ein.";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {
		initializeOxygenInterfaceVariables(authorAccess);

		String fileName = getFileName();
		String fileNameJustNumber = OperationUtility.subStringWithRegex(fileName, "(?<=habwbrf_)(.*)");

		AttrValue teiID = new AttrValue(fileName);
		AttrValue bodyID = new AttrValue("body_" + fileNameJustNumber);
		AuthorElement teiNode = getTeiNode();
		AuthorElement bodyNode = getBodyNode();

		if (fileName != "" && teiNode != null && bodyNode != null) {
			documentController.setAttribute("xml:id", teiID, teiNode);
			documentController.setAttribute("xml:id", bodyID, bodyNode);
		} else {
			workspaceAccess.showErrorMessage(
					"Schwerer Fehler!\n" + "Eines oder mehrere essentielle Dokumentskomponten fehlen.\n"
							+ "Bitte wenden Sie sich an die technische Betreuung.");
		}
	}

	private String getFileName() {

		AuthorDocument document = documentController.getAuthorDocumentNode();
		String fullFileName = document.getSystemID();

		// Regex sucht nach Dateinamen ohne Pfad und ohne Endung
		String fileName = OperationUtility.subStringWithRegex(fullFileName, "(?!.*/)(.*)(?=\\.xml)");

		return fileName;
	}

	private AuthorElement getTeiNode() throws AuthorOperationException {
		String bodyXPath = "//*[local-name()='TEI']";

		AuthorNode[] teiNodes = documentController.findNodesByXPath(bodyXPath, true, true, true);

		if (teiNodes.length > 0) {
			return (AuthorElement) teiNodes[0];
		} else {
			return null;
		}
	}

	private AuthorElement getBodyNode() throws AuthorOperationException {

		String teiXPath = "//*[local-name()='body']";

		AuthorNode[] teiNodes = documentController.findNodesByXPath(teiXPath, true, true, true);

		if (teiNodes.length > 0) {
			return (AuthorElement) teiNodes[0];
		} else {
			return null;
		}
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

}
