package de.historischekommission_muenchen.oxydition.brfedition.common.operations;

import de.historischekommission_muenchen.oxydition.common.beans.InsertDateBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.InsertDateDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorConstants;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Abstrakte Oberklasse für Operationen, bei denen Datums-Metadaten in den
 * <teiHeader> eines Briefes eingefügt werden; Ruft Dialogfenster für Eingabe
 * auf, konstuiert <date>-Element und führt Einfügung aus; Erbende Klassen
 * müssen das type-Atrribut des <correspAction>-Elements festlegen, in das das
 * Datum eingefügt werden soll. Die Angabe kann dabei als unsicher markiert
 * werden.
 * 
 * @author Maximilian Schrott
 *
 */
public abstract class InsertMetadataDate extends OxyditionOperation {

	protected String insertLocation;

	@Override
	public String getDescription() {
		return null;
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap argss)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		setInsertLocation();

		String xPathLocation = "//correspAction[@type='" + insertLocation + "']";

		InsertDateDialog dlg = new InsertDateDialog(oxygenFrame, "Metadaten für Versanddatum eingeben");

		InsertDateBean input = dlg.displayDialog();

		if (input.isFilled) {
			insertMetadataDate(input, xPathLocation);
		}
	}

	private String constructDateFragment(InsertDateBean input) {

		StringBuilder fragmentBuilder = new StringBuilder();

		fragmentBuilder.append("<date " + XMLUtility.TEIXMLNS + " when='" + input.normDate + "'");

		if (input.isUncertain) {
			fragmentBuilder.append(" cert='low'");
		}

		String readDate = XMLUtility.escapeXML(input.readDate);
		fragmentBuilder.append(">" + readDate + "</date>");

		return fragmentBuilder.toString();
	}

	private void insertMetadataDate(InsertDateBean input, String xPathLocation) throws AuthorOperationException {
		String sentDateFragment = constructDateFragment(input);
		documentController.insertXMLFragmentSchemaAware(sentDateFragment, xPathLocation,
				AuthorConstants.POSITION_INSIDE_LAST);

	}

	/** Methode zum Anpassen des Einfügeorts **/
	protected abstract void setInsertLocation();

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

}
