package de.historischekommission_muenchen.oxydition.brfedition.common.operations;

import java.io.FileNotFoundException;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SimpleListSelectionDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.ReadRegisterOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorConstants;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Oxygen-Author-Operation mit die Benutzer*innen einen Verweis auf einen
 * Eintrag aus der Registerdatenbank für Sachschlagwörter, der den ganzen Brief
 * beschreibt, in das <textClass>/<keyword>-Element des <teiHeaders> eine
 * Briefes einfügen können. Dabei können sie die Angabe als unsicher markieren.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertMetadataKeyword extends ReadRegisterOperation {

	private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {
			new ArgumentDescriptor(ARGUMENT_URL, ArgumentDescriptor.TYPE_STRING, urlArgDescription) };

	@Override
	public String getDescription() {
		return "Öffnet ein Dialogfenster, in dem die Benutzer*innen einen Eintrag aus der Sachschlagworts-Registerdatenbank auswählen können. Diese Schlagwort wird, samt RDB-Verweis in das <textclass>/<keywords>-Element im teiHeader des Briefes eingefügt";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		if (!validateUrlArgument(args)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {
			ReadRegisterBasic registerReader = getRegisterEntries(args);

			String[] eintraege = registerReader.getEintrag();
			String[] ids = registerReader.getID();

			SimpleListSelectionDialog dlg = new SimpleListSelectionDialog(oxygenFrame,
					"Sachschlagwort-Metadateneintrag hinzufügen", eintraege, ids);
			ListSelectionBean input = dlg.displayDialog();

			if (input.isFilled) {
				insertMetadataKeywordFragment(input);
			}

		} catch (FileNotFoundException e) {
			displayRegisterNotFoundMessage();
		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			OperationUtility.handleGenericException(oxygenFrame, e);
		}

	}

	private void insertMetadataKeywordFragment(ListSelectionBean input) throws AuthorOperationException {
		String keywordFragment = constructMetadataKeywordFragment(input);
		String insertLocation = "//textClass/keywords";
		documentController.insertXMLFragment(keywordFragment, insertLocation, AuthorConstants.POSITION_INSIDE_LAST);
	}

	private String constructMetadataKeywordFragment(ListSelectionBean input) {

		String keywordTerm = XMLUtility.escapeXML(input.value);
		keywordTerm = keywordTerm.replaceAll("- ", "");

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<term " + XMLUtility.TEIXMLNS + " type='Schlagwort' ");
		fragmentBuilder.append("corresp='" + input.id + "'>" + keywordTerm + "</term>");

		return fragmentBuilder.toString();
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

}
