package de.historischekommission_muenchen.oxydition.brfedition.common.operations;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionWithCertaintyBean;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.InsertFromRegisterIntoLocationWithCertainty;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einen Eintrag aus der
 * Personenregisterdatenbank auswählen und als Empfänger in das
 * <correspAction @type="received">-Element des <teiHeader>s eines Briefes
 * einfügen können. Dabei können sie die Angabe als unsicher markieren.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertMetadataReceiver extends InsertFromRegisterIntoLocationWithCertainty {

	@Override
	public String getDescription() {
		return "Öffnet ein Dialogfenster, in dem die Benutzer*innen einen Personeneintrag aus der Registerdatenbank heraussuchen können. "
				+ "Die gewählte Person wird im Metadatenapparat des TEI-Headers als Empfänger des Briefes hinterlegt.";
	}

	@Override
	protected void initializeVariables() {
		nodeVal = "//*[local-name()='listPerson']/*[local-name()='person']";
		// Ein geschütztes Leerzeichen als Trenner zwischen Name und Qualifikator, um
		// letzteren unauffällig weglassen und beide Elemente sicher von einander zu
		// trennen zu können
		expressionVal = "//*[local-name()='persName' and @type='Registername'] +\"\u00A0\"+//*[local-name()='note' and @type='Qualifikator']";
		idVal = "@xml:id";

		dialogTitle = "Metadaten für Empfänger eingeben";
		xPathLocation = "//correspAction[@type='received']";

	}

	@Override
	protected String constructEntityFragment(ListSelectionBean input) {
		StringBuilder fragmentBuilder = new StringBuilder();

		fragmentBuilder.append("<persName " + XMLUtility.TEIXMLNS + " key='" + input.id + "'");

		if (((ListSelectionWithCertaintyBean) input).isUncertain) {
			fragmentBuilder.append(" cert='low'");
		}

		// Name von Qualifikator trennen
		String inputValue = input.value;
		String inputName = inputValue.split("\u00A0")[0];
		inputName = XMLUtility.escapeXML(inputName);

		fragmentBuilder.append(">" + inputName + "</persName>");

		return fragmentBuilder.toString();
	}

}
