package de.historischekommission_muenchen.oxydition.common.dialogs;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

import de.historischekommission_muenchen.oxydition.common.beans.AutoTagBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.elements.SelectionListSearchFieldListener;
import de.historischekommission_muenchen.oxydition.common.utilities.DialogUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.OxygenDocumentTextSearcher;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.access.AuthorEditorAccess;
import ro.sync.ecss.extensions.api.content.TextContentIterator;
import ro.sync.exml.view.graphics.Rectangle;

/**
 * Dialogfenster für Operationen, die das halbautomatischen Referenzieren von
 * Entitäten im Text ermöglichen. Das Dialogfenster benutzt einen TextSearcher,
 * um das Dokument nach allen Vorkommen eine übergebenen Textausschnitts zu
 * durchsuchen. Bei jedem Treffer haben die Benutzer*innen die Möglichkeit
 * diesen als false positive zu überspringen oder ihn als einen von bis zu drei
 * voreingestellten Referenztypen vorzumerken. Das Dialogfenster enthält
 * außerdem eine Auswahlliste aus der die Benutzer*innen einen
 * Registerdatenbankeintrag auswählen können. Nach Abschluss der Eingabe werden
 * die Informationen des gewählten Eintrags und die vorgemerkten Textstellen an
 * die aufrufende Klasse zurückgegeben.
 * 
 * @author Maximilian Schrott
 *
 */
public class AutoTagDialog extends ModularListSelectDialog {

	private static final long serialVersionUID = -8184461780061523227L;

	private JPanel functionButtonPanel;
	private JTextField selectedTextField;

	private TextContentIterator iterator;
	private OxygenDocumentTextSearcher textSearcher;
	private AuthorDocumentController documentController;
	private AuthorEditorAccess editorAccess;

	private JButton tagButtonOne;
	private JButton tagButtonTwo;
	private JButton tagButtonThree;
	private JButton skipButton;

	private AutoTagBean autoTagInputBean;
	private String generalType;
	private String selectedText;

	public AutoTagDialog(JFrame oxygenFrame, String[] eintraege, String[] ids, AuthorAccess authorAccess,
			Object[] typeArguments, String selectedText) {

		super(oxygenFrame, "Automatische Referenzierung", eintraege, ids);

		String frameTitle = constructFrameTitle(typeArguments);
		setTitle(frameTitle);
		this.selectedText = selectedText;
		documentController = authorAccess.getDocumentController();
		editorAccess = authorAccess.getEditorAccess();

		insertFunctionButtonPanel(typeArguments);
		initializeSelectedTextField(selectedText);
		initializeOkButton();

		updateSearchRegisterFieldWithoutDoctorTitle(selectedText);
		initializeTextSearch(authorAccess, selectedText);
	}

	private String constructFrameTitle(Object[] typeArguments) {

		Object generalTypeArgVal = typeArguments[0];
		generalType = (String) typeArguments[1];

		if (OperationUtility.isValidArgument(generalTypeArgVal)) {
			generalType = (String) generalTypeArgVal;
		}

		return "Automatische " + generalType + "-Referenzierung";
	}

	private void insertFunctionButtonPanel(Object[] typeArguments) {
		functionButtonPanel = new JPanel();
		functionButtonPanel.setLayout(new BoxLayout(functionButtonPanel, BoxLayout.Y_AXIS));
		functionButtonPanel.setPreferredSize(new Dimension(180, 0));
		functionButtonPanel.setBorder(new EmptyBorder(19, 0, 0, 0));

		assembleFunctionButtonPanel(typeArguments);

		mainContent.add(functionButtonPanel, BorderLayout.LINE_END);
	}

	// Fügt abhängig von den gesetzten Argumenten ein bis drei Referenztyp-Buttons
	// auf der rechten Seite des Fensters ein
	private void assembleFunctionButtonPanel(Object[] typeArguments) {

		tagButtonOne = insertTagButton(typeArguments[1]);
		if (OperationUtility.isValidArgument(typeArguments[2])) {
			tagButtonTwo = insertTagButton(typeArguments[2]);
			if (OperationUtility.isValidArgument(typeArguments[3])) {
				tagButtonThree = insertTagButton(typeArguments[3]);
			}
		}

		functionButtonPanel.add(Box.createRigidArea(new Dimension(0, 16)));
		skipButton = insertButton("Überspringen", functionButtonPanel);
		skipButton.addActionListener(new SkipButtonListener());
	}

	private JButton insertTagButton(Object typeArgument) {
		String tagButtonText = "<" + (String) typeArgument + ">";
		JButton button = insertButton(tagButtonText, functionButtonPanel);
		button.addActionListener(new TagButtonListener((String) typeArgument));
		functionButtonPanel.add(DialogUtility.BIGRIGID);

		return button;
	}

	private void initializeSelectedTextField(String selectedText) {
		selectedTextField.setText(selectedText);
		selectedTextField.setEnabled(false);
	}

	private void initializeOkButton() {
		okButton.setText("Referenzierungen übernehmen");
		okButton.setEnabled(false);
	}

	private void initializeTextSearch(AuthorAccess authorAccess, String selectedText) {

		documentController = authorAccess.getDocumentController();
		int documentStartOffset = documentController.getAuthorDocumentNode().getStartOffset();
		int documentEndOffset = documentController.getAuthorDocumentNode().getEndOffset();

		iterator = documentController.getTextContentIterator(documentStartOffset, documentEndOffset);

		textSearcher = new OxygenDocumentTextSearcher(iterator, selectedText);
	}

	@Override
	protected void assembleListPanel() {
		listPanel = new JPanel();
		listPanel.setLayout(new BoxLayout(listPanel, BoxLayout.Y_AXIS));

		selectedTextField = insertTextFieldwithLabel("Markierter Textausschnitt", listPanel);
		listPanel.add(DialogUtility.BIGRIGID);

		searchRegisterField = insertTextFieldwithLabel("Registerdatenbank durchsuchen", listPanel);
		listPanel.add(DialogUtility.BIGRIGID);

		insertRegisterListWithLabel("Registerdatenbankeintrag auswählen");
		Document searchRegisterDocument = searchRegisterField.getDocument();
		searchRegisterDocument.addDocumentListener(new SelectionListSearchFieldListener(selectionList));
	}

	@Override
	public AutoTagBean displayDialog() {

		autoTagInputBean = new AutoTagBean();
		okButton.addActionListener(new AutoTagOkButtonListener());
		readyDialog();

		return autoTagInputBean;
	}

	@Override
	public void readyDialog() {

		int firstOffset = textSearcher.findNextTermOffset();

		// Zum Anfang des Dokuments scrollen (verhindert unerwünschtes Verhalten bei der
		// Anzeige von Suchtreffern am Anfang des Dokuments)
		Rectangle documentStartRectangle = new Rectangle(0, 0, 0, 0);
		editorAccess.scrollToRectangle(documentStartRectangle);

		try {
			if (firstOffset >= 0) {

				if (isTaggablePosition(firstOffset)) {
					shiftOxygenViewtoOffset(firstOffset);
					finalizeDialog();
				} else {
					if (continueSearch()) {
						finalizeDialog();
					}
				}
			} else {
				cancelOperation(true);
			}

		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		}
	}

	private void finalizeDialog() {
		getContentPane().add(mainContent);
		pack();
		setSize(500, 400);

		// Dialog rechts oben in (Haupt-)Bildschirm setzen
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation(screenSize.width - 550, 50);

		setVisible(true);
	}

	private void cancelOperation(boolean automatic) {

		if (automatic) {
			JOptionPane.showMessageDialog(getParent(),
					"Die Operation hat das Ende des Dokuments erreicht ohne einen unmarkierten Treffen zu finden.\n"
							+ "Die Operation wird beendet.",
					"Operation beendet", JOptionPane.INFORMATION_MESSAGE);
			dispose();
		} else {
			int confirmation = JOptionPane.showConfirmDialog(getParent(),
					"Soll die Operation abgebrochen und alle Referenzierungen verworfen werden?",
					"Operation abbrechen?", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);

			if (confirmation == JOptionPane.YES_OPTION) {
				dispose();
			}
		}

	}

	private void bookmarkTagging(String type) throws BadLocationException {

		int tagLocationOffset = textSearcher.getLastFoundOffset();

		// Prüfen ob bereits ein Eintrag aus der RDB-Liste ausgewählt wurde
		if (selectionList.getSelectedIndex() != -1) {

			autoTagInputBean.addOffsetToTag(tagLocationOffset, type);

			// RDB-Liste und -Suchfeld sperren und "Übernehmen"-Button freischalten sobald
			// der erste Treffer vorgemerkt wurde
			searchRegisterField.setEnabled(false);
			selectionList.setEnabled(false);
			okButton.setEnabled(true);

			continueSearch();

		} else {
			JOptionPane.showMessageDialog(getParent(),
					"Bitte wählen Sie aus der Registerdatenbanksliste den Eintrag aus, der refernziert werden soll.",
					"Kein RDB-Eintrag gewählt", JOptionPane.WARNING_MESSAGE);

		}

	}

	private boolean continueSearch() throws BadLocationException {
		int nextOffset;

		boolean cont = true;

		while (cont) {

			nextOffset = textSearcher.findNextTermOffset();

			if (nextOffset >= 0) {

				if (isTaggablePosition(nextOffset)) {
					shiftOxygenViewtoOffset(nextOffset);
					return true;
				}

			} else {
				cont = false;
			}
		}

		endOfSearch();
		return false;

	}

	private boolean isTaggablePosition(int offset) throws BadLocationException {
		return !editorAccess.isOffsetInInvisibleBounds(offset)
				&& !XMLUtility.offsetInSelfOrDescendantOfElement(offset, "rs", documentController)
				&& !XMLUtility.offsetInSelfOrDescendantOfElement(offset, "bibl", documentController)
				&& !XMLUtility.offsetInSelfOrDescendantOfElement(offset, "persName", documentController)
				&& !XMLUtility.offsetInSelfOrDescendantOfElement(offset, "placeName", documentController)
				&& !XMLUtility.offsetSurroundedByElement(offset, "term", documentController);
	}

	private void shiftOxygenViewtoOffset(int nextOffset) {

		Rectangle searchHitRectangle = editorAccess.modelToViewRectangle(nextOffset);

		searchHitRectangle.y = searchHitRectangle.y + 350;
		editorAccess.scrollToRectangle(searchHitRectangle);
		editorAccess.select(nextOffset, nextOffset + selectedText.length());

	}

	private void endOfSearch() {

		disableFunctionButtons();

		if (autoTagInputBean.offsetsToTagList.isEmpty()) {
			cancelOperation(true);
		} else {
			finalizeOperation(true);
		}

	}

	private void disableFunctionButtons() {
		disableFunctionButton(tagButtonOne);
		disableFunctionButton(tagButtonTwo);
		disableFunctionButton(tagButtonThree);
		disableFunctionButton(skipButton);
	}

	private void disableFunctionButton(JButton button) {

		if (button != null) {
			button.setEnabled(false);
		}
	}

	private void finalizeOperation(boolean automatic) {
		StringBuilder confirmStringBuilder = new StringBuilder();

		if (automatic) {
			confirmStringBuilder.append(
					"Die Operation hat das Ende des Dokuments erreicht ohne weitere (unmarkierte) Treffer zu finden.\n");
		}

		int numberOfOffsetsToTag = autoTagInputBean.offsetsToTagList.size();

		if (numberOfOffsetsToTag == 1) {
			confirmStringBuilder.append("1 Textstelle wird als " + generalType + " markiert");
		} else {
			confirmStringBuilder.append(numberOfOffsetsToTag + " Textstellen werden als " + generalType + " markiert");
		}

		int confirmation = JOptionPane.showConfirmDialog(getParent(), confirmStringBuilder.toString(),
				"Operation abschließen?", JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);

		if (confirmation == JOptionPane.YES_OPTION) {
			autoTagInputBean.value = selectedText;
			autoTagInputBean.id = selectionList.getMetaEntryAt(selectionList.getSelectedIndex());
			autoTagInputBean.isFilled = true;
			dispose();
		}
	}

	private class TagButtonListener implements ActionListener {

		String tagType;

		public TagButtonListener(String tagType) {
			this.tagType = tagType;
		}

		public void actionPerformed(ActionEvent e) {

			try {
				bookmarkTagging(tagType);
			} catch (BadLocationException e1) {
				displayBadLocationMessage(e1);
			}
		}

	}

	private class SkipButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			try {
				continueSearch();
			} catch (BadLocationException e1) {
				displayBadLocationMessage(e1);
			}
		}
	}

	private class AutoTagOkButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {

			finalizeOperation(false);
		}
	}
}
