package de.historischekommission_muenchen.oxydition.common.dialogs;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JTextField;

import de.historischekommission_muenchen.oxydition.common.beans.InsertDateBean;
import de.historischekommission_muenchen.oxydition.common.utilities.DialogUtility;

/**
 * Dialogfenster für die Eingabe von Datumsangaben in normaler und
 * maschinenlesbarer Form. Enthält zwei entsprechende Texteingabefelder und ein
 * Checkbox für die Kennzeichnung unsicherer angaben.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertDateDialog extends ModularGridBagDialog {

	private static final long serialVersionUID = -2690854854668147111L;

	InsertDateBean input;

	JTextField readDateField;
	JTextField normDateField;
	JCheckBox certainCheck;

	public InsertDateDialog(JFrame oxygenFrame, String frameTitle) {
		super(oxygenFrame, frameTitle);

		readDateField = addField("Datum");
		normDateField = addField("Formatiertes Datum (YYYY-MM-DD)");
		certainCheck = addCheckBox("Datum unsicher");

		insertOperationEndButtonPanel("Übernehmen", "Abbrechen");

	}

	public InsertDateBean displayDialog() {
		input = new InsertDateBean();
		okButton.addActionListener(new OkButtonListener());

		readyDialog();
		return input;
	}

	protected class OkButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {

			boolean inputValid = validateInput();

			if (inputValid) {
				extractInput();
				dispose();
			}
		}

		private boolean validateInput() {

			boolean allValid = true;
			String notAllValidMessage = "";

			if (!DialogUtility.isFilledTextfield(readDateField)) {
				allValid = false;
				notAllValidMessage = "Kein Wert für Datum eingegeben; \n";
			}

			if (!DialogUtility.isFilledTextfield(normDateField)) {
				allValid = false;
				notAllValidMessage = "Kein Wert für formatiertes Datum eingeben;";
			}

			if (allValid) {
				return true;
			} else {
				displayInputNotAllValidMessage(notAllValidMessage);
				return false;
			}
		}

		private void extractInput() {

			input.readDate = readDateField.getText();
			input.normDate = normDateField.getText();
			input.isUncertain = certainCheck.isSelected();
			input.isFilled = true;
		}
	}
}
