package de.historischekommission_muenchen.oxydition.common.dialogs;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JFrame;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionWithCertaintyBean;

/**
 * Dialogfenster für Operationen, bei denen die Benutzer*innen einen Eintrag aus
 * einer Liste auswählen müssen und zusätzlich markieren können, dass
 * einzufügende Angabe unsicher ist.
 * 
 * @author Maximilian Schrott
 *
 */
public class ListSelectionWithCertaintyDialog extends ModularListSelectDialog {

	private static final long serialVersionUID = -2987305060209393365L;

	private ListSelectionWithCertaintyBean input;
	private JCheckBox checkCertainty;

	public ListSelectionWithCertaintyDialog(JFrame owner, String title, String[] eintraege, String[] ids) {
		super(owner, title, eintraege, ids);
		checkCertainty = insertCheckBoxWithText("Angabe nicht sicher", additionalInputPanel);
	}

	@Override
	public ListSelectionWithCertaintyBean displayDialog() {

		input = new ListSelectionWithCertaintyBean();
		okButton.addActionListener(new OkButtonListener());

		readyDialog();
		return input;

	}

	protected class OkButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {

			int selectedIndex = selectionList.getSelectedIndex();

			if (selectedIndex > -1) {
				extractInput(selectedIndex);
				dispose();
			}

		}

		private void extractInput(int selectedIndex) {

			String selectedValue = selectionList.getSelectedValue();
			String selectedId = selectionList.getMetaEntryAt(selectedIndex);
			boolean isUncertain = checkCertainty.isSelected();

			input.value = selectedValue;
			input.id = selectedId;
			input.isUncertain = isUncertain;
			input.isFilled = true;
		}

	}
}
