package de.historischekommission_muenchen.oxydition.common.dialogs;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.text.Document;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.elements.OxyditionInputFilter;
import de.historischekommission_muenchen.oxydition.common.dialogs.elements.SelectionListSearchFieldListener;
import de.historischekommission_muenchen.oxydition.common.dialogs.elements.UpdatingJList;
import de.historischekommission_muenchen.oxydition.common.utilities.DialogUtility;

/**
 * Modulare Dialogfenstervorlage, für Operationen bei denen eine Auswahlliste
 * benötigt wird.. Die Vorlage ist darauf ausgelegt, durch erbende Klassen in
 * Aufbau und Funktion an die jeweiligen Anforderungen angepasst zu werden.
 * Hierfür können nach Bedarf Elemente wie Textfelder, TextAreas oder Checkboxes
 * zum Dialogfenster hinzugefügt werden. Ebenso kann das Verhalten der
 * Funktionsabschlussbuttons angepasst werden. Standartmäßig werden die Daten
 * des gewählten Listeneintrags zurückgegeben.
 * 
 * @author Maximilian Schrott
 *
 */

public class ModularListSelectDialog extends OxyditionOperationDialog {

	private static final long serialVersionUID = 1L;

	protected JPanel centerPanel;
	protected JPanel listPanel;

	protected JPanel additionalInputPanel;
	protected UpdatingJList<String> selectionList;

	protected JTextField searchRegisterField;

	protected String[] eintraege;
	protected String[] ids;

	private static final Dimension LABELMAXSIZE = new Dimension(5000, 20);
	private static final Dimension TEXTFIELDMAXSIZE = new Dimension(5000, 25);
	private static final Dimension TEXTAREAMAXSIZE = new Dimension(5000, 300);

	private float leftAlign = Component.LEFT_ALIGNMENT;
	private int dialogVSize = 400;

	protected ListSelectionBean inputBean;

	public ModularListSelectDialog(JFrame owner, String title, String[] eintraege, String[] ids) {
		super(owner, title);

		this.eintraege = eintraege;
		this.ids = ids;

		assembleMainContent();
	}

	public ModularListSelectDialog(JFrame owner, String title, String[] eintraege) {
		super(owner, title);

		this.eintraege = eintraege;
		this.ids = new String[eintraege.length];

		assembleMainContent();
	}

	public ListSelectionBean displayDialog() {

		inputBean = new ListSelectionBean();
		okButton.addActionListener(new OkButtonListener());

		readyDialog();
		return inputBean;
	}

	protected void assembleMainContent() {

		insertCenterPanel();
		insertOperationEndButtonPanel("Einfügen", "Abbrechen");
	}

	private void insertCenterPanel() {

		centerPanel = new JPanel(new BorderLayout(5, 5));
		insertListPanel();
		mainContent.add(centerPanel, BorderLayout.CENTER);
		insertAdditionalInputPanel();
	}

	private void insertListPanel() {
		assembleListPanel();
		centerPanel.add(listPanel, BorderLayout.CENTER);
	}

	protected void assembleListPanel() {

		listPanel = new JPanel();
		listPanel.setLayout(new BoxLayout(listPanel, BoxLayout.Y_AXIS));
		searchRegisterField = insertTextFieldwithLabel("Liste filtern:", listPanel);
		OxyditionInputFilter.setForInput(searchRegisterField);
		listPanel.add(DialogUtility.BIGRIGID);
		insertSelectionList();

		Document searchRegisterDocument = searchRegisterField.getDocument();
		searchRegisterDocument.addDocumentListener(new SelectionListSearchFieldListener(selectionList));
	}

	private void insertAdditionalInputPanel() {

		additionalInputPanel = new JPanel();
		additionalInputPanel.setLayout(new BoxLayout(additionalInputPanel, BoxLayout.Y_AXIS));
		centerPanel.add(additionalInputPanel, BorderLayout.PAGE_END);
	}

	protected void insertSelectionList() {

		selectionList = new UpdatingJList<String>(eintraege, ids);
		JScrollPane registerScroll = new JScrollPane();
		registerScroll.setViewportView(selectionList);
		registerScroll.setAlignmentX(leftAlign);
		registerScroll.setMaximumSize(new Dimension(5000, 250));
		listPanel.add(registerScroll);
	}

	protected void insertRegisterListWithLabel(String labelText) {
		JLabel label = constructLabel(labelText);
		listPanel.add(label);
		listPanel.add(DialogUtility.SMALLRIGID);
		insertSelectionList();
	}

	/**
	 * Fügt ein neues Textfeld mit einem darüber positionierten Label in das
	 * übergebene Panel ein und gibt es zurück.
	 */
	public JTextField insertTextFieldwithLabel(String labelText, JPanel panel) {
		JLabel label = constructLabel(labelText);

		panel.add(label);
		panel.add(DialogUtility.SMALLRIGID);
		JTextField textField = constructTextField();
		OxyditionInputFilter.setForInput(textField);
		panel.add(textField);
		dialogVSize += 50;

		return textField;
	}

	/**
	 * Fügt eine neue TextArea mit einem darüber positionierten Label in das
	 * übergebene Panel ein und gibt sie zurück.
	 */
	public JTextArea insertTextAreaWithLabel(String labelText, JPanel panel) {

		JLabel label = constructLabel(labelText);
		panel.add(label);
		JTextArea textArea = constructTextArea();
		OxyditionInputFilter.setForInput(textArea);
		panel.add(textArea);
		dialogVSize += 250;

		return textArea;
	}

	/**
	 * Fügt eine neue Checkbox mit einem daneben positionierten Label in das
	 * übergebene Panel ein und gibt sie zurück.
	 */
	public JCheckBox insertCheckBoxWithText(String text, JPanel panel) {

		JCheckBox checkBox = new JCheckBox(text);
		panel.add(checkBox);
		dialogVSize += 50;

		return checkBox;
	}

	/**
	 * Fügt einen Button mit Beschriftung in das übergeben Panel ein und gibt ihn
	 * zurück
	 */
	public JButton insertButton(String text, JPanel panel) {

		JButton button = new JButton(text);
		panel.add(button);

		return button;
	}

	protected JLabel constructLabel(String text) {

		JLabel label = new JLabel(text);
		label.setAlignmentX(leftAlign);
		label.setMaximumSize(LABELMAXSIZE);

		return label;
	}

	protected JTextField constructTextField() {

		JTextField textField = new JTextField();
		textField.setAlignmentX(leftAlign);
		textField.setMaximumSize(TEXTFIELDMAXSIZE);

		return textField;
	}

	protected JTextArea constructTextArea() {

		JTextArea textArea = new JTextArea();
		textArea.setAlignmentX(leftAlign);
		textArea.setMaximumSize(TEXTAREAMAXSIZE);
		textArea.setLineWrap(true);
		textArea.setWrapStyleWord(true);

		Border textAreaBorder = BorderFactory.createEtchedBorder(EtchedBorder.LOWERED);
		textArea.setBorder(textAreaBorder);

		return textArea;
	}

	/**
	 * Erlaubt es den Inhalt des Registersuchfelds auf den übergebenen Wert zu
	 * verändern
	 */
	public void updateSearchRegisterField(String text) {
		searchRegisterField.setText(text);
	}

	/**
	 * Erlaubt es den Inhalt des Registersuchfelds auf den übergebenen Wert zu
	 * ändern. Beginnt der übergebene Text mit der Zeichenfolge "Dr." wird diese
	 * vorher abgeschnitten.
	 * 
	 */
	public void updateSearchRegisterFieldWithoutDoctorTitle(String text) {
		text = deleteDoctorTitle(text);
		searchRegisterField.setText(text);
	}

	private String deleteDoctorTitle(String text) {

		if (text.startsWith("Dr.")) {
			String modifiedText = text.substring(3);
			modifiedText = modifiedText.trim();
			return modifiedText;
		} else {
			return text;
		}
	}

	// Richtet das Dialogfenster ein und macht es sichtbar
	protected void readyDialog() {

		getContentPane().add(mainContent);
		pack();
		setSize(950, dialogVSize);
		setLocationRelativeTo(getOwner());
		setVisible(true);
	}

	@Override
	protected void cancelButtonAction() {
		dispose();

	}

	protected class OkButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			inputBean = extractInput();
			dispose();
		}
	}

	protected ListSelectionBean extractInput() {
		ListSelectionBean bean = new ListSelectionBean();

		int selectedIndex = selectionList.getSelectedIndex();

		if (selectedIndex > -1) {
			bean.id = selectionList.getMetaEntryAt(selectedIndex);
			bean.isFilled = true;
		}
		return bean;
	}
}
