package de.historischekommission_muenchen.oxydition.common.dialogs.elements;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import de.historischekommission_muenchen.oxydition.common.utilities.StandardReadRegister;

/**
 * Hilfsklasse, die von einer Oxygen-Author-Operation mit den Zugangsdaten für
 * einen ReadRegisterzugriff an ein Dialogfenster übergeben wird. Dieses kann
 * sie dann aufrufen, um den Zugriff durchzuführen und die ermittelten Daten
 * zurückzugeben.
 * 
 * @author Maximilian Schrott
 *
 */
public class ReadRegisterStorer {

	private String nodeVal;
	private String expressionVal;
	private String idVal;

	/**
	 * 
	 * @param node
	 *            -Die XPath-Beschreibung des Knotentypen, der referenziert werden
	 *            soll, + z.B.
	 *            "//*[local-name()='listPerson']/*[local-name()='person']"
	 * @param expression
	 *            - Die Form des Vorschlags, der den Benutzer*innen in der Registerliste
	 *            angezeigt wird. Muss aus Subelementen der oben ausgewählten
	 *            Basisknoten bestehen. Strings stehen ind "", Attribute beginnen
	 *            mit @, Elemente mit /, Subelemente mit // und XPath-Ausdrücke mit
	 *            .), z.B. //*[local-name()='persName' and type='Registername'] + "
	 *            " +//*[local-name()='note' and @type='Qualifikator']"
	 * @param id
	 *            - der Wert der für Rückgabe ausgelesen werden soll. Syntax wie bei
	 *            expression
	 */
	public ReadRegisterStorer(String node, String expression, String id) {
		
		nodeVal = node;
		expressionVal = expression;
		idVal = id;
	}

	/**
	 * Erstellt ein ReadRegister-Objekt aus den übergebenene Parametern und gibt ihn
	 * zurück
	 * 
	 * @param url
	 *            - globale URL der zu durchsuchenden Datei
	 * @return - das generierte ReadRegisterObjekt
	 * @throws SAXException 
	 * @throws IOException 
	 * @throws ParserConfigurationException 
	 * @throws XPathExpressionException 
	 */
	public ReadRegisterBasic getListData(String url) throws XPathExpressionException, ParserConfigurationException, IOException, SAXException {

		String urlVal = "file:" + /*urlBaseVal +*/ "/" + url;
		return new StandardReadRegister(urlVal, nodeVal, expressionVal, idVal);
	}

}
