package de.historischekommission_muenchen.oxydition.common.operations;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Random;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SimpleListSelectionDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.ReadRegisterOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.KeywordReadRegister;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Oxygen-Author-Operation, mit der die Benutzer*innen einen (beliebig großen)
 * markierten Textaussschnitt mit einem <index>/<anchor>-Elementpaar umgeben
 * können, das mit einer Sachschlagworts-Registerdatenbank verknüpft ist.
 * 
 * @author Maximilian Schrott
 *
 */
public class EmbedKeywordNested extends ReadRegisterOperation {

	private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {
			new ArgumentDescriptor(ARGUMENT_URL, ArgumentDescriptor.TYPE_STRING, urlArgDescription) };

	@Override
	public String getDescription() {
		return "Fügt um einen markierten Text ein Paar aus <index>- und <anchor>-Elementen ein, "
				+ "die miteinander verknüpft sind und auf einen Eintrag"
				+ " im geschachtelten Sachschlagwortsregister verweisen,"
				+ " den die Benutzer*innen in einem Dialogfenster auswählen können.";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		if (!validateUrlArgument(args)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {
			ReadRegisterBasic registerReader = getRegisterEntries(args);

			String[] eintraege = registerReader.getEintrag();
			String[] ids = registerReader.getID();

			SimpleListSelectionDialog dlg = new SimpleListSelectionDialog(oxygenFrame, "Keyword hinterlegen", eintraege,
					ids);
			ListSelectionBean input = dlg.displayDialog();

			if (input.isFilled) {
				embedKeywordIntoDocument(input);
			}

		} catch (FileNotFoundException e) {
			displayRegisterNotFoundMessage();
		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			OperationUtility.handleGenericException(oxygenFrame, e);
		}
	}

	private void embedKeywordIntoDocument(ListSelectionBean input) throws AuthorOperationException {

		int randomNum = new Random().nextInt(9000) + 1000;
		String anchorId = input.id + "_" + randomNum;

		String indexFragment = constructIndexFragment(input, anchorId);
		String anchorFragment = constructAnchorFragment(anchorId);

		int selectionStart = editorAccess.getSelectionStart();
		int selectionEnd = editorAccess.getSelectionEnd();

		documentController.insertXMLFragment(anchorFragment, selectionEnd);
		documentController.insertXMLFragment(indexFragment, selectionStart);

	}

	private String constructIndexFragment(ListSelectionBean input, String anchorId) {

		StringBuilder fragmentBuilder = new StringBuilder();

		fragmentBuilder.append("<index " + XMLUtility.TEIXMLNS + " spanTo='#" + anchorId + "' ");
		fragmentBuilder.append("indexName='Sachschlagwörter' ");
		fragmentBuilder.append("corresp='" + input.id + "'>");

		String value = XMLUtility.escapeXML(input.value);
		fragmentBuilder.append("<term " + XMLUtility.TEIXMLNS + ">" + value + "</term>");
		fragmentBuilder.append("</index>");

		return fragmentBuilder.toString();
	}

	private String constructAnchorFragment(String anchorId) {

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<anchor " + XMLUtility.TEIXMLNS + " xml:id='" + anchorId + "' ");
		fragmentBuilder.append("type='Sachschlagwörter'/>");

		return fragmentBuilder.toString();
	}

	@Override
	protected ReadRegisterBasic getRegisterEntries(ArgumentsMap args)
			throws XPathExpressionException, ParserConfigurationException, IOException, SAXException {
		String urlArgString = (String) args.getArgumentValue(ARGUMENT_URL);

		return new KeywordReadRegister(urlArgString);
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

}
