package de.historischekommission_muenchen.oxydition.common.operations;

import java.io.File;
import java.util.ArrayList;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SimpleListSelectionDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.FileListOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einen Verweis auf einen
 * Datei (als <ref>-Element) einfügen können. Die Operation verweist nur auf den
 * Dateinamen. Dieser dient als Platzhalter und muss zu einem späteren Zeitpunkt
 * (automatisiert) durch den finalen Namen des referenzierten Dokuments ersetzt
 * werden.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertFileReference extends FileListOperation {

	@Override
	public String getDescription() {
		return "Fügt an der Position des Textcursors einen Verweis(platzhalter) auf eine Dokumentendatei im Ordner des aktuellen Dokuments ein.";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {
		initializeOxygenInterfaceVariables(authorAccess);

		ArrayList<File> fileList = getFileList(args);
		String[] documentFileNames = getFileNames(fileList);

		// Dummy-Element für die Id-Liste, die vom Dialogfenster erwartet, für die
		// Operation aber nicht benötigt wird
		String[] dummyIds = new String[documentFileNames.length];

		SimpleListSelectionDialog dlg = new SimpleListSelectionDialog(oxygenFrame, "Dokumentenreferenz einfügen",
				documentFileNames, dummyIds);
		ListSelectionBean input = dlg.displayDialog();

		if (input.isFilled) {
			insertDocumentFileReferenceFragment(input);
		}
	}

	private void insertDocumentFileReferenceFragment(ListSelectionBean input) throws AuthorOperationException {
		String fragment = constructReferenceFragment(input);
		documentController.insertXMLFragment(fragment, caretPosition);
	}

	protected String constructReferenceFragment(ListSelectionBean input) {
		StringBuilder referenceBuilder = new StringBuilder();
		String value = XMLUtility.escapeXML(input.value);

		referenceBuilder.append("<ref " + XMLUtility.TEIXMLNS + " type='Dateireferenz' target='" + value + "'/>");

		return referenceBuilder.toString();
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

}
