package de.historischekommission_muenchen.oxydition.common.operations;

import de.historischekommission_muenchen.oxydition.common.beans.DoubleListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.InsertDoubleListReference;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen eine Referenzen auf eine
 * Anmerkung im Editionstext einfügen können. Da Anmerkungen zum
 * Bearbeitungszeitpunkt noch keine verwendbare Nummer haben, muss stattdessen
 * das xml:id-Attribut benutzt werden. Um den Benutzer*innen den Zugriff hierauf
 * möglichst einfach zu machen, wählen sie die entsprechende Anmerkung anhand
 * ihres Textes aus. Hierfür suchen sie zuerst aus einer Liste der
 * Editionsdokumentendateien diejenige aus, die die gesuchte Anmerkung enthält.
 * Daraufhin bekommen sie, in einer zweiten Liste, alle Fußnoten des Dokument
 * mit ihrem jeweiligen Text(anfang) als Vorschlag angezeigt. Der eingefügte
 * Verweis ist ein <ref>-Element mit der ID der referenzieren Fußnote als
 * target-Attribut. Er muss zu einem späteren Zeitpunkt durch die finale Nummer
 * der Anmerkung ersetzt werden.
 * 
 * 
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertNoteReference extends InsertDoubleListReference {

	@Override
	public String getDescription() {
		return "Erlaubt es den Benutzer*innen einen Verweise auf eine Anmerkung einzufügen, indem sie zuerst eine Datei auswählen und dann eine Anmerkung anhand ihres Textes.";
	}

	@Override
	protected String constructReferenceFragment(DoubleListSelectionBean input) {
		StringBuilder fragmentBuilder = new StringBuilder();

		fragmentBuilder.append("<ref " + XMLUtility.TEIXMLNS + " type='Anmerkungsreferenz' ");
		fragmentBuilder.append("target='" + input.listTwoID + "'/>");

		return fragmentBuilder.toString();
	}

	@Override
	protected void initializeSecondListVariables() {
		secondListNode = "//*[local-name()='note']";
		secondListExpression = ".";
		secondListID = "@xml:id";

	}

	@Override
	protected String getFrameTitle() {
		return "Anmerkungsreferenz einfügen";
	}

}
