package de.historischekommission_muenchen.oxydition.common.operations;

import de.historischekommission_muenchen.oxydition.common.beans.DoubleListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.InsertDoubleListReference;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen eine Referenzen auf einen
 * Absatz mit xml:id-Attribut oder einen als Referenztextstelle referenzierten
 * Textausschnitt einfügen können (diese Markierungen müssen zuvor ggfs. durch
 * eine andere Operation gesetzt worden sein). Hierfür suchen sie zuerst aus
 * einer Liste der Editionsdokumentendateien diejenige aus, die den gesuchten
 * Absatz oder die gesuchte Textstelle enthält. Daraufhin bekommen sie, in einer
 * zweiten Liste, alle referenzierbaren Absätze und Textstellen im gewählten
 * Dokument angezeigt, mit ihrem jeweiligen Text(anfang) als Vorschlag. Der
 * eingefügte Verweis ist ein <ref>-Element mit der ID des referenzieres
 * Absatzs/ der referenzierten Textstelle als target-Attribut. Er muss zu einem
 * späteren Zeitpunkt durch eine nachvollziehbare Angabe, z.B. die Seitenzahl,
 * ersetzt werden.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertParagraphOrTextPositionReference extends InsertDoubleListReference {

	@Override
	public String getDescription() {
		return "Erlaubt es den Benutzer*innen einen Verweis auf einen (referenzierbaren) Absatz oder einen (referenzierbaren) Textausschnitt einzufügen, indem sie zuerst eine Datei auswählen und dann einen Absatz anhand seines Textes.";
	}

	@Override
	protected String constructReferenceFragment(DoubleListSelectionBean input) {
		StringBuilder fragmentBuilder = new StringBuilder();

		fragmentBuilder.append("<ref " + XMLUtility.TEIXMLNS + " type='Textreferenz' ");
		fragmentBuilder.append("target='" + input.listTwoID + "'/>");

		return fragmentBuilder.toString();
	}

	@Override
	protected void initializeSecondListVariables() {
		secondListNode = "//*[(local-name()='p' and @id) or (local-name()='seg' and @type='Referenztextstelle')]";
		secondListExpression = ".";
		secondListID = "@xml:id";

	}

	@Override
	protected String getFrameTitle() {
		return "Textstelle referenzieren";
	}

}
