package de.historischekommission_muenchen.oxydition.common.operations;

import de.historischekommission_muenchen.oxydition.common.beans.DoubleListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.InsertDoubleListReference;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen eine Referenzen auf einen
 * Absatz mit xml:id-Attribut einfügen können (ggfs. muss die xml:id zuerst mit
 * einer anderen Operation gesetzt werden). Hierfür suchen sie zuerst aus einer
 * Liste der Editionsdokumentendateien diejenige aus, die den gesuchten Absatz
 * enthält. Daraufhin bekommen sie, in einer zweiten Liste, alle Absätze mit
 * xml:id im gewählten Dokument angezeigt, mit ihrem jeweiligen Text(anfang) als
 * Vorschlag. Der eingefügte Verweis ist ein <ref>-Element mit der ID des
 * referenzieres Absatzs als target-Attribut. Er muss zu einem späteren
 * Zeitpunkt durch eine nachvollziehbare Angabe, z.B. die Seitenzahl, ersetzt
 * werden.
 * 
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertParagraphReference extends InsertDoubleListReference {

	@Override
	public String getDescription() {
		return "Erlaubt es den Benutzer*innen einen Verweis auf einen (referenzierbaren) Absatz einzufügen, indem sie zuerst eine Datei auswählen und dann einen Absatz anhand seines Textes.";
	}

	@Override
	protected String constructReferenceFragment(DoubleListSelectionBean input) {
		StringBuilder fragmentBuilder = new StringBuilder();

		fragmentBuilder.append("<ref " + XMLUtility.TEIXMLNS + " type='Absatzreferenz' ");
		fragmentBuilder.append("target='" + input.listTwoID + "'/>");

		return fragmentBuilder.toString();
	}

	@Override
	protected void initializeSecondListVariables() {
		secondListNode = "//*[local-name()='p' and @id]";
		secondListExpression = ".";
		secondListID = "@xml:id";

	}

	@Override
	protected String getFrameTitle() {
		return "Absatz referenzieren";
	}

}
