package de.historischekommission_muenchen.oxydition.common.operations;

import de.historischekommission_muenchen.oxydition.common.beans.DoubleListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.InsertDoubleListReference;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;

/**
 * Operation mit der die Benutzer*innen eine Referenz auf einen Unterabschnitt
 * oder einen Textunterpunkt innerhalb einer ausgewählten Datei einfügen können.
 * Hierfür suchen sie zuerst aus einer Liste der Editionsdokumentendateien
 * diejenige aus, die den gesuchten Textteil enthält. Daraufhin bekommen sie, in
 * einer zweiten Liste, alle Unterabschnitte und Unterpunkte mit ihrer
 * jeweiligen Überschrift als Vorschlag angezeigt. Der eingefügte Verweis ist
 * ein <ref>-Element mit der ID des referenzieres Textteils als target-Attribut.
 * Er muss zu einem späteren Zeitpunkt durch eine nachvollziehbare Angabe
 * ersetzt werden.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertTextDivReference extends InsertDoubleListReference {

	@Override
	protected String constructReferenceFragment(DoubleListSelectionBean input) {
		StringBuilder fragmentBuilder = new StringBuilder();

		fragmentBuilder.append("<ref " + XMLUtility.TEIXMLNS + " type='Textteilreferenz' ");
		fragmentBuilder.append("target='" + input.listTwoID + "'/>");

		return fragmentBuilder.toString();
	}

	@Override
	protected void initializeSecondListVariables() {
		secondListNode = "//*[@type='Unterabschnitt' or @type='Textunterpunkt']";
		secondListExpression = "//*[local-name()='head']";
		secondListID = "@xml:id";
	}

	@Override
	protected String getFrameTitle() {
		return "Textteil referenzieren";
	}

	@Override
	public String getDescription() {
		return "Erlaubt es den Benutzer*innen einen Verweis auf einen Unterabschnitt oder Textunterpunkt einzufügen, indem sie zuerst eine Datei auswählen und dann ein Textteil anhand seines Textes.";
	}

}
