package de.historischekommission_muenchen.oxydition.common.operations;

import javax.swing.JFrame;
import javax.swing.text.BadLocationException;

import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import de.historischekommission_muenchen.oxydition.common.beans.InsertStructureElementBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.InsertStructureElementDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.InsertStructureElement;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.protedition.common.utilities.ProtocolUtility;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einen Textunterpunkt ins
 * Dokument einfügen können. Hierfür müssen sie in einem Dialogfenster die
 * Unterpunktsnummer eingeben. Ein Unterpunkt ist ein <div> Element mit
 * type-Attribut="Textunterpunkt" und einer xml:id, die sich aus der
 * Dokumenten-ID und der Unterpunktnummer ergibt. Die Operation fügt innerhalb
 * des neuen Textunterpunkts außerdem einen Textteilpunkt ein.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertTextSubPoint extends InsertStructureElement {

	@Override
	public String getDescription() {
		return "Ermöglicht es den Benutzer*innen einen Textunterpunkt in einen Textabschnitt einer Edition einzufügen.";
	}

	@Override
	protected InsertStructureElementBean getInputData(JFrame oxygenFrame, String dialogTitle) {

		InsertStructureElementDialog dlg = new InsertStructureElementDialog(oxygenFrame, "Textunterpunkt einfügen",
				false);
		InsertStructureElementBean input = dlg.displayDialog();

		return input;
	}

	@Override
	protected String constructTopicXMLFragment(InsertStructureElementBean input, String idPrefix) {

		String normNum = normaliseNormNum(input.normNum);

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<div " + XMLUtility.TEIXMLNS + " ");
		fragmentBuilder.append("type='Textunterpunkt' ");
		fragmentBuilder.append("xml:id='" + idPrefix + "g" + normNum + "'>");
		fragmentBuilder.append(constructHeadXMLFragment(input));
		fragmentBuilder.append("<div type='Textteilpunkt'/>");
		fragmentBuilder.append("</div>");

		return fragmentBuilder.toString();

	}

	@Override
	protected String constructHeadXMLFragment(InsertStructureElementBean input) {

		StringBuilder headBuilder = new StringBuilder();
		headBuilder.append("<head " + XMLUtility.TEIXMLNS + ">");

		// Angezeigte Nummer optional
		if (!input.elementNum.isEmpty()) {

			String elementNum = XMLUtility.escapeXML(input.elementNum);
			headBuilder.append("<num " + XMLUtility.TEIXMLNS + ">" + elementNum + "</num>");
		}
		headBuilder.append(input.title + "</head>");

		return headBuilder.toString();

	}

	@Override
	protected boolean isCaretPositionValid(int caretPosition, AuthorDocumentController documentController)
			throws BadLocationException {

		AuthorElement surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

		return (XMLUtility.elementTypeEquals(surroundNode, "Textabschnitt")
				|| XMLUtility.elementTypeEquals(surroundNode, "Quelldokument")
				|| XMLUtility.elementTypeEquals(surroundNode, "Unterabschnitt")
				|| XMLUtility.elementTypeEquals(surroundNode, "Textunterpunkt"));

	}

	@Override
	protected String findIdPrefix(int caretPosition, AuthorDocumentController documentController)
			throws BadLocationException {

		// Präfix ist entweder die ProtokollID (wenn auf oberster
		// Gliederungsebende) oder der übergeordnete Textgliederungspunkt
		AuthorElement surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

		if (XMLUtility.elementTypeEquals(surroundNode, "Textabschnitt")
				|| XMLUtility.elementTypeEquals(surroundNode, "Quelldokument")) {
			AttrValue protIdVal = ProtocolUtility.getProtocolID(documentController);

			return protIdVal.getRawValue();
		} else {

			AttrValue surroundNodeIDVal = surroundNode.getAttribute("xml:id");
			return surroundNodeIDVal.getRawValue();
		}
	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "An dieser Position darf kein Textunterpunkt eingefügt werden.\n"
				+ "Fügen Sie Textunterpunkt bitte nur auf der obersten Gliederungsebene oder direkt innerhalb eines Unterabschnitts ein.";
	}

}
