package de.historischekommission_muenchen.oxydition.common.operations.abstracts;

import java.io.File;
import java.util.ArrayList;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionTextfieldBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.ListSelectionTextfieldDialog;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;

/**
 * Abstrakte Oberklasse für Operationen, mit denen die Benutzer*innen neue Dokumente
 * zu einer Editionsgesamtdatei hinzufügen können. Die Operation öffnet ein
 * Dialogfenster, in dem alle noch nicht in der Gesamtdatei referenzierten
 * XML-Dateien aus dem spezifizierten Verzeichnis (siehe Parameter in
 * Oberklasse)Dokumentsverzeichnis angezeigt werden. Die Benutzer*innen können eines
 * davon auswählen und geben zusätzlich die Nummer ein, mit der das Dokument
 * eingefügt werden soll. Die Form des eingefügten XML-Elements muss in den
 * erbenden klassen implementiert werden.
 * 
 * @author Maximilian Schrott
 *
 */
public abstract class FullEditionInsertDocument extends FileListOperation {

	@Override
	public String getDescription() {
		return null;
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {
		initializeOxygenInterfaceVariables(authorAccess);

		ArrayList<File> fileList = getFileList(args);
		String[] allDocumentFileNames = getFileNames(fileList);
		String[] filteredFileNames = filterDocumentFileNames(allDocumentFileNames);

		ListSelectionTextfieldDialog dlg = new ListSelectionTextfieldDialog(oxygenFrame, "Dokument einfügen",
				filteredFileNames, "Dokumentennummer");
		dlg.prefillTextfield("Nr.");
		ListSelectionTextfieldBean input = dlg.displayDialog();

		if (input.isFilled) {

			String fragment = constructDocumentFragment(input);
			documentController.insertXMLFragment(fragment, caretPosition);
		}
	}

	private String[] filterDocumentFileNames(String[] allFileNames) throws AuthorOperationException {

		AuthorNode[] alreadyIncludedFileNodes = documentController.findNodesByXPath("//xi:include[@href]", false, false,
				false);

		String[] alreadyIncludedFileNames = getIncludedFileNames(alreadyIncludedFileNodes);
		String[] filteredFileNames = filterFileNames(allFileNames, alreadyIncludedFileNames);

		return filteredFileNames;
	}

	private String[] getIncludedFileNames(AuthorNode[] includedFileNodes) {

		String[] includedDocumentNames = new String[includedFileNodes.length];
		for (int i = 0; i < includedFileNodes.length; i++) {

			AuthorElement documentElement = (AuthorElement) includedFileNodes[i];
			String hrefValue = XMLUtility.getAttributeRawValue(documentElement, "href");
			includedDocumentNames[i] = hrefValue;
		}

		return includedDocumentNames;
	}

	private String[] filterFileNames(String[] allFileNames, String[] includedFileNames) {

		ArrayList<String> filteredFileNamesList = new ArrayList<String>();

		for (int i = 0; i < allFileNames.length; i++) {
			String documentFileName = allFileNames[i];

			boolean isMatch = false;

			for (int j = 0; j < includedFileNames.length; j++) {
				String includedFileName = includedFileNames[j];

				if (includedFileName.equalsIgnoreCase(documentFileName)) {
					isMatch = true;
				}
			}
			if (!isMatch) {
				filteredFileNamesList.add(documentFileName);
			}
		}

		String[] filteredFileNames = new String[filteredFileNamesList.size()];
		filteredFileNames = filteredFileNamesList.toArray(filteredFileNames);

		return filteredFileNames;
	}

	protected abstract String constructDocumentFragment(ListSelectionTextfieldBean input);

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

}
