package de.historischekommission_muenchen.oxydition.common.operations.abstracts;

import java.io.File;
import java.util.ArrayList;

import de.historischekommission_muenchen.oxydition.common.beans.DoubleListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.DoubleListSelectionDialog;
import de.historischekommission_muenchen.oxydition.common.dialogs.elements.ReadRegisterStorer;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Abstrakte Oberklasse für Operationen, mit denen die Benutzer*innen eine
 * Referenz auf ein XML-Element mittels zweier Auswahllisten erstellen können.
 * Die erste Liste zeigt dabei eine Auwahl von XML-Dateien an, die untere Liste,
 * nach Auswahl einer Datei, alle in ihr vorkommenden Elemente, des
 * spezifizierten Typs. Bei Abschluss der Operation wird ein neues XML-Fragment
 * mit einer referenz auf das gewählte Element an der Position des Textcursors
 * eingesetzt
 * 
 * @author Maximilian Schrott
 *
 */
public abstract class InsertDoubleListReference extends FileListOperation {

	protected String secondListNode;
	protected String secondListExpression;
	protected String secondListID;

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {
		initializeOxygenInterfaceVariables(authorAccess);
		initializeSecondListVariables();

		ArrayList<File> fileList = getFileList(args);
		String documentFileNames[] = getFileNames(fileList);
		String[] documentFilePaths = getFilePaths(fileList);

		ReadRegisterStorer updater = new ReadRegisterStorer(secondListNode, secondListExpression, secondListID);

		DoubleListSelectionDialog dlg = new DoubleListSelectionDialog(oxygenFrame, getFrameTitle(), updater,
				documentFileNames, documentFilePaths);

		DoubleListSelectionBean input = dlg.displayDialog();

		if (input.isFilled) {
			insertReferenceFragment(input);
		}
	}

	protected void insertReferenceFragment(DoubleListSelectionBean input) throws AuthorOperationException {
		String referenceFragment = constructReferenceFragment(input);
		documentController.insertXMLFragment(referenceFragment, caretPosition);

	}

	@Override
	// Überschreibt Methode um nur XML-Dateien zuzulassen
	protected ArrayList<File> filterFiles(File[] files) {
		ArrayList<File> fileredFileList = new ArrayList<File>();

		for (int i = 0; i < files.length; i++) {
			File file = files[i];
			String fileName = file.getName();

			if (!file.isDirectory() && !fileName.startsWith(".") && fileName.endsWith(".xml")) {
				fileredFileList.add(file);
			}
		}

		return fileredFileList;
	}

	protected abstract String constructReferenceFragment(DoubleListSelectionBean input);

	protected abstract void initializeSecondListVariables();

	protected abstract String getFrameTitle();

}
