package de.historischekommission_muenchen.oxydition.common.operations.abstracts;

import java.io.FileNotFoundException;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionWithCertaintyBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.ListSelectionWithCertaintyDialog;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Spezialisierung von InsertFromRegisterIntoLocation für Operationen, bei denen
 * die Benutzer*innen zusätzlich markieren können, dass die getroffenen Angaben unsicher
 * sind.
 * 
 * @author Maximilian Schrott
 *
 */
public abstract class InsertFromRegisterIntoLocationWithCertainty extends InsertFromRegisterIntoLocation {

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);
		initializeVariables();

		if (!validateArguments(args)) {
			displayInvalidArgumentsMessage();
			return;
		}
		try {

			ReadRegisterBasic registerReader = getRegisterEntries(args);

			String[] eintraege = registerReader.getEintrag();
			String[] ids = registerReader.getID();

			ListSelectionWithCertaintyDialog dlg = new ListSelectionWithCertaintyDialog(oxygenFrame, dialogTitle,
					eintraege, ids);
			ListSelectionWithCertaintyBean input = dlg.displayDialog();

			if (input.isFilled) {
				insertFromRegister(input);
			}

		} catch (FileNotFoundException e) {
			displayRegisterNotFoundMessage();
		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			e.printStackTrace();
			OperationUtility.handleGenericException(oxygenFrame, e);
		}

	}

}
