package de.historischekommission_muenchen.oxydition.common.operations.abstracts;

import java.io.FileNotFoundException;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionTextfieldBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.ListSelectionTextfieldDialog;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * 
 * Abstrakte Überklasse für Operationen, die ein mit dem Register verbundenes
 * XML-Fragment an der Position des Textcursors einfügen und den Benutzer*innen zu
 * Eingabe einen ListSelectioNTextfieldDialog zur Verfügung stellen.
 * 
 * @author Maximilian Schrott
 *
 */
public abstract class InsertFromRegisterWithTextfield extends ReadRegisterOperationJustURL {

	// Weitere Operationsvariablen
	protected String dialogTitle;
	protected String textFieldLabel;

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);
		initializeVariables();

		if (!validateArguments(args)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {
			ReadRegisterBasic registerReader = getRegisterEntries(args);

			String[] eintraege = registerReader.getEintrag();
			String[] ids = registerReader.getID();

			ListSelectionTextfieldDialog dlg = new ListSelectionTextfieldDialog(oxygenFrame, dialogTitle, eintraege,
					ids, textFieldLabel);
			ListSelectionTextfieldBean input = dlg.displayDialog();

			if (input.isFilled) {
				insertFromRegister(input);
			}

		} catch (FileNotFoundException e) {
			displayRegisterNotFoundMessage();
		} catch (IOException | XPathExpressionException | ParserConfigurationException | SAXException e) {
			OperationUtility.handleGenericException(oxygenFrame, e);
		}

	}

	protected void insertFromRegister(ListSelectionTextfieldBean input) throws AuthorOperationException {
		String entityFragment = constructEntityFragment(input);
		documentController.insertXMLFragment(entityFragment, caretPosition);
	}

	/**
	 * Erstellt das XML-Fragment, dass in da Dokument eingefügt werden soll
	 * 
	 * @param input
	 *            - Datenhaltungsobjekt mit den Benutzereingaben
	 * @return - das fertige XMLFragment
	 */
	protected abstract String constructEntityFragment(ListSelectionTextfieldBean input);

}
