package de.historischekommission_muenchen.oxydition.common.operations.abstracts;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.text.BadLocationException;

import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperation;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.access.AuthorEditorAccess;
import ro.sync.ecss.extensions.api.access.AuthorWorkspaceAccess;

/**
 * Abstakte Oberklasse für alle oxygen-Author-Operationen im Oxydition-Projekt.
 * Stellt einige regelmäßig benötigte Mehtoden und Variablen zur Verfügung.
 * 
 * @author Maximilian Schrott
 *
 */
public abstract class OxyditionOperation implements AuthorOperation {

	protected AuthorWorkspaceAccess workspaceAccess;
	protected AuthorEditorAccess editorAccess;
	protected AuthorDocumentController documentController;

	protected JFrame oxygenFrame;
	protected int caretPosition;

	/*
	 * protected static String urlArgDescription =
	 * "Die URL der Registerdatenbanksdatei," +
	 * "\nz.B.: \"file:${pd}\\Bearbeitung\\RDB\\${pb}RDB_PER.xml"; protected static
	 * String nodeArgDescription =
	 * "Die XPath-Beschreibung des Knotentypen, der refernziert werden soll," +
	 * "z.B. \"//*[local-name()='listPerson']/*[local-name()='person']\""; protected
	 * static String expressionArgDescription =
	 * "Die Form des Vorschlags, der den Benutzer*innen in der Registerliste angezeigt wird."
	 * + "\n Muss aus Subelementen der oben ausgewählten Basisknoten bestehen." +
	 * "\n Strings stehen ind \"\", Attribute beginnen mit @, Elemente mit /, Subelemente mit // und XPath-Ausdrücke mit .),"
	 * +
	 * "\n \"//*[local-name()='persName' and type='Registername'] + \" \" +//*[local-name()='note' and @type='Qualifikator']\""
	 * ;
	 */
	@Override
	public abstract String getDescription();

	@Override
	public abstract void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException;

	@Override
	public abstract ArgumentDescriptor[] getArguments();

	/**
	 * Führt alle grundlegenden Schritte aus, die bei den meisten Operationen
	 * benötigt werden
	 */
	@SuppressWarnings("unused")
	protected void initializeOxygenInterfaceVariables(AuthorAccess authorAccess) {

		workspaceAccess = authorAccess.getWorkspaceAccess();
		editorAccess = authorAccess.getEditorAccess();
		documentController = authorAccess.getDocumentController();

		JFrame oxygenFrame = (JFrame) workspaceAccess.getParentFrame();
		caretPosition = editorAccess.getCaretOffset();
	}

	/**
	 * Standartisiertes Fehlerverhalten bei BadLocationExceptions: Bringt den
	 * Stacktrace auf die Konsole und öffnet eine JOptionPane mit einer
	 * Fehlermeldung für die Benutzer*innen.
	 */
	protected void displayBadLocationMessage(BadLocationException e) {

		String badLocationTitle = "Schwerer Fehler";
		String badLocationMessage = "Schwerer Fehler: BADLOCATIONEXCEPTION";

		e.printStackTrace();
		JOptionPane.showMessageDialog(oxygenFrame, badLocationMessage, badLocationTitle, JOptionPane.WARNING_MESSAGE);

	}

	/**
	 * Standartisiertes Fehlerverhalten für den Fall, dass ein Operation, die einen
	 * markierten Textausschnitt erwartet keinen solchen findet: Zeigt eine
	 * entsprechende Fehlermeldung an
	 */
	protected void displayNoSelectionMessage() {
		String noSelectionTitle = "Kein markierter Text";
		String noSelectionMessage = "Zum Ausführen der Operation muss ein Textausschnitt markiert werden";

		JOptionPane.showMessageDialog(oxygenFrame, noSelectionMessage, noSelectionTitle, JOptionPane.WARNING_MESSAGE);
	}

	/**
	 * Standartisiertes Fehlerverhalten für den Fall, dass nicht alle
	 * Operationsargumente gültig sind: Zeigt eine entsprechende Fehlermeldung an
	 */
	protected void displayInvalidArgumentsMessage() {
		String invalidArgumentsTitle = "Ungültige Funktionsargumente";
		String invalidArgumentsMessage = "Fehler: Eines oder mehrere für die Operation benötigten Argumente fehlen oder sind nicht korrekt eingerichtet.";

		JOptionPane.showMessageDialog(oxygenFrame, invalidArgumentsMessage, invalidArgumentsTitle,
				JOptionPane.WARNING_MESSAGE);
	}

	/**
	 * Standartisiertes Fehlerverhalten für den Fall, dass die Position des
	 * Textcursors beim Auslösen der Operation ungültig ist
	 */

	protected void displayInvalidCaretPositionMessage() {
		String invalidCaretPositionTitle = "Ungültige Textcursorposition";
		JOptionPane.showMessageDialog(oxygenFrame, getInvalidCaretPositionMessage(), invalidCaretPositionTitle,
				JOptionPane.WARNING_MESSAGE);
	}

	protected Object getInvalidCaretPositionMessage() {
		return "Die Operation kann an der aktuellen Operation des Textcursors nicht ausgeführt werden";
	}
}
