package de.historischekommission_muenchen.oxydition.common.utilities;

import java.awt.Component;
import java.awt.Dimension;

import javax.swing.Box;
import javax.swing.JTextArea;
import javax.swing.JTextField;

/**
 * Stellt Funktionen für den Gestaltung von Dialogfenstern und
 * Dialogfensterverhalten zur Verfügung
 * 
 * @author Maximilian Schrott
 *
 */
public class DialogUtility {

	public static final Component BIGRIGID = Box.createRigidArea(new Dimension(0, 8));
	public static final Component SMALLRIGID = Box.createRigidArea(new Dimension(0, 4));

	/** Prüft ob es im übergebenen Textfeld eine Benutzereingabe gibt */
	public static boolean isFilledTextfield(JTextField textField) {
		String inputText = textField.getText();
		return !inputText.isEmpty();
	}

	/** Prüft ob es in der übergebenen Textarea eine Benutzereingabe gibt */
	public static boolean isFilledTextArea(JTextArea textArea) {
		String inputText = textArea.getText();
		return !inputText.isEmpty();
	}

	/**
	 * Prüft ob das übergebene Texteingabefeld ein maschinenlesbar formatiertes
	 * Datum enthält
	 */
	public static boolean isValidNormalizedDateInput(JTextField normalizedDateInputField) {
		String normalizedDate = normalizedDateInputField.getText();
		return normalizedDate.matches("[0-9]{4}-((0[0-9])|(1[0-2]))-(([0-2][0-9])|(3[0-1]))");
	}

	/**
	 * Prüft ob das übergebene Texteingabefeld eine maschienenlesbare Zeitangabe
	 * enthält
	 */
	public static boolean isValidNormalizedTimeInput(JTextField normalizedDateInputField) {
		String normalizedTime = normalizedDateInputField.getText();
		return normalizedTime.matches("(([0-1][0-9])|(2[0-3])):[0-5][0-9]");
	}

	/**
	 * Prüft ob das übergebene Texteingabefeld eine korrekte (dezimale)
	 * Breitengradangabe ist
	 */
	public static boolean isValidLatitude(JTextField latitudeInputField) {
		String latitudeString = latitudeInputField.getText();
		float latitudeFloat = -1000;

		try {
			latitudeFloat = Float.parseFloat(latitudeString);
		} catch (NumberFormatException e) {
		}

		return latitudeFloat <= 90 && latitudeFloat >= -90;

	}

	/**
	 * Prüft ob das übergebene Texteingabefeld eine korrekte (dezimale)
	 * Längengradangabe ist
	 */
	public static boolean isValidLongitude(JTextField longitudeInputField) {
		String latitudeString = longitudeInputField.getText();
		float longitudeFloat = -1000;

		try {
			longitudeFloat = Float.parseFloat(latitudeString);
		} catch (NumberFormatException e) {
		}

		return longitudeFloat <= 180 && longitudeFloat >= -180;

	}
}
