
package de.historischekommission_muenchen.oxydition.common.utilities;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.codec.binary.Base64;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Eine Anwendungs-Klasse zum durchsuchen und auswerten der XML-Registerdatei
 * für ungedruckte Quellen nach dem Muster der MinRProt. Der Klasse
 * org.bbaw.telote.ediarum.ReadRegister nachempfunden
 * 
 * @author Maximilian Schrott
 * 
 */
public class UnpublishedSourceReadRegister extends ReadRegisterBasic {

	/**
	 * Konstruktor
	 * 
	 * @param indexURI - URI der Registerdatei
	 * @throws ParserConfigurationException
	 * @throws IOException
	 * @throws SAXException
	 * @throws XPathExpressionException
	 */
	public UnpublishedSourceReadRegister(String indexURI)
			throws ParserConfigurationException, IOException, SAXException, XPathExpressionException {

		// Zur Auswertung der XML-Datei vorbereiten
		DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
		domFactory.setNamespaceAware(false);
		DocumentBuilder builder = domFactory.newDocumentBuilder();
		// Wenn es sich um eine URL mit Authentifizierung handelt, ..
		InputStream is;
		if (indexURI.indexOf('@') > -1) {
			// .. werden die Verbindungsdaten gelesen ..
			String authString = indexURI.substring(indexURI.indexOf("://") + 3, indexURI.indexOf('@'));
			String webPage = indexURI.substring(0, indexURI.indexOf("://") + 3)
					+ indexURI.substring(indexURI.indexOf('@') + 1);
			byte[] authEncBytes = Base64.encodeBase64(authString.getBytes());
			String authStringEnc = new String(authEncBytes);
			// .. und eine Verbindung mit Login geöffnet.
			URL url = new URL(webPage);
			URLConnection urlConnection = url.openConnection();
			urlConnection.setRequestProperty("Authorization", "Basic " + authStringEnc);
			is = urlConnection.getInputStream();
		} else {
			// Im anderen Fall wird direkt eine Verbindung geöffnet.
			URL url = new URL(indexURI);
			URLConnection urlConnection = url.openConnection();
			is = urlConnection.getInputStream();
		}

		// Dann wird die Datei gelesen
		InputSource inputSource = new InputSource(is);
		Document indexDoc = builder.parse(inputSource);
		// Die xPath-Routinen werden vorbereitet
		XPath xpath = XPathFactory.newInstance().newXPath();

		// Querytext zum finden der Quellen-Einträge
		String nodeQuery = "//*[local-name()='list' and @type='Quellen']/*[local-name()='item']";

		// Das XPath-Query wird definiert.
		XPathExpression expr = xpath.compile(nodeQuery);

		// Die Resultate werden ausgelesen
		Object result = expr.evaluate(indexDoc, XPathConstants.NODESET);
		NodeList registerNodes = (NodeList) result;
		// .. dann werden für die Einträge und IDs entsprechend lange
		// Arrays
		// angelegt.
		eintrag = new String[registerNodes.getLength()];
		id = new String[registerNodes.getLength()];

		// Auswertung jedes einzelnen Quelleneintrags
		for (int i = 0; i < registerNodes.getLength(); i++) {

			// das Quellen-<item-selbst
			Element currentElement = (Element) registerNodes.item(i);

			// hinterlegte ID speichern
			id[i] = currentElement.getAttribute("xml:id");

			/*
			 * Anzeigetext aus Quellen-Identifieren und Kennnamen der übergeordneten
			 * Sammlungen erzeugen
			 */
			String eintragString = "";

			// <item>-der übergeordneten Sammlung (Archib/Bestand)
			Element parentCollection = (Element) currentElement.getParentNode().getParentNode();
			// Übergeordnete Sammlungsliste (@type="Archive" bzw.
			// "Bestände")
			Element parentCollectionList = (Element) parentCollection.getParentNode();

			// Wenn die übergeordnete Sammlung ein Bestand ist, auch
			// noch deren übergeordnete Sammlung (das Archiv) auswerten
			// und zum Anzeigetext hinzufügen
			if (parentCollectionList.getAttribute("type").equalsIgnoreCase("Bestände")) {

				Element archive = (Element) parentCollectionList.getParentNode();

				eintragString += getNormName(archive) + ": ";
			}

			// Kennname der übergeordneten Sammlung hinzufügen
			eintragString += getNormName(parentCollection) + "\u00A0";

			// <title>-Element des Quelleintrags herausfinden und zu
			// Anzeigetext hinzufügen
			NodeList currentChildren = currentElement.getChildNodes();

			for (int j = 0; j < currentChildren.getLength(); j++) {

				// Kindknoten die keine Elemente sind herausfiltern
				if (currentChildren.item(j).getNodeType() == Node.ELEMENT_NODE) {
					Element child = (Element) currentChildren.item(j);

					if (child.getNodeName().equalsIgnoreCase("title")) {

						eintragString += child.getTextContent();
					}

				}
			}

			eintrag[i] = eintragString;
		}
	}

	private String getNormName(Element sourceCollection) {

		// Kindknoten der Sammlung auswerten
		NodeList archiveChildren = sourceCollection.getChildNodes();

		for (int i = 0; i < archiveChildren.getLength(); i++) {

			Node childNode = archiveChildren.item(i);

			// Kindknoten die keine Elementknoten sind herazsfiltern
			if (childNode.getNodeType() == Node.ELEMENT_NODE) {

				Element childElement = (Element) childNode;

				if (childElement.getNodeName().equalsIgnoreCase("name")
						&& childElement.getAttribute("type").equalsIgnoreCase("Kennname")) {

					String collectionName = childElement.getTextContent();

					return collectionName;
				}
			}
		}

		return "FEHLER";
	}
}
