package de.historischekommission_muenchen.oxydition.protedition.common.dialogs;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JTextField;

import de.historischekommission_muenchen.oxydition.common.dialogs.ModularGridBagDialog;
import de.historischekommission_muenchen.oxydition.common.utilities.DialogUtility;
import de.historischekommission_muenchen.oxydition.protedition.common.beans.InsertProtocolTopicBean;

/**
 * Dialogfenster für Operationen bei denen ein neuer Tagesordnungspunkt in ein
 * Protokoll eingefügt wird. Wird mittels Operationsargumenten auf die
 * Anforderungen der Operation angepasst von der es aufgerufen wird. Enthält
 * dementsprechend unterschiedliche Textfelder und Checkboxes.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertProtocolTopicDialog extends ModularGridBagDialog {

	private static final long serialVersionUID = -1679900207508546108L;

	public static final int MINRPROT_TOP = 0;
	public static final int MINRPROT_SUBTOP = 1;
	public static final int WEIMPROT_TOP = 2;
	public static final int WEIMPROT_SUBTOP = 3;

	private int dialogType;
	private JTextField topNumField;
	private JTextField normNumField;
	private JTextField titleField;
	private JCheckBox checkAsInSource;
	private JCheckBox checkAddedNumber;
	private JCheckBox checkAddedTitle;
	private JCheckBox checkAddedAll;
	private JCheckBox checkMovedTitle;
	private JCheckBox checkMovedAll;
	private JCheckBox checkHiddenTop;
	private ButtonGroup checkBoxGroup;

	private InsertProtocolTopicBean inputBean;

	/**
	 * 
	 * @param owner      - Fenster des Oxygen-Editors
	 * @param dialogType - Der Typ der Einfügeoperation: MINRPROT_TOP,
	 *                   MINRPROT_SUBTOP, WEIMPROT_TOP, WEIMPROT_SUBTOP
	 */
	public InsertProtocolTopicDialog(JFrame owner, int dialogType) {

		super(owner);

		this.dialogType = dialogType;

		setTitle(selectDialogTitle(dialogType));

		addInputFields();

		checkBoxGroup = new ButtonGroup();

		addAsInSourceCheckBox();

		// Checkboxes abhängig von der aufrufenden Operation hinzufügen
		if (dialogType == 0 || dialogType == 1) {
			addMinRPRotCheckboxes();
		} else if (dialogType == 2 || dialogType == 3) {
			addWeiMProtCheckboxes();
		}

		if (dialogType == 1) {
			addHiddenTopicCheckbox();
		}

		insertOperationEndButtonPanel("Übernehmen", "Abbrechen");
	}

	// Setzt den Dialogtitel auf Basis der aufrufenden Operation ein
	private String selectDialogTitle(int dialogType) {

		if (dialogType == 0 || dialogType == 2) {
			return "Tagesordnungspunkt einfügen";
		} else {
			return "Tagesordnungsunterpunkt einfügen";
		}
	}

	private void addInputFields() {
		topNumField = addField("TOP-Nummer");
		normNumField = addField("TOP-Nummer (arabische Ziffern/Großbuchstaben)");
		titleField = addField("Überschrift");
		addEmptySpace();
	}

	private void addAsInSourceCheckBox() {
		checkAsInSource = addCheckBox("TOP wie in Vorlage");
		checkBoxGroup.add(checkAsInSource);
		checkAsInSource.setSelected(true);
		addEmptySpace();
	}

	private void addMinRPRotCheckboxes() {
		checkAddedNumber = addCheckBox("TOP-Nummer weicht von Quelle ab");
		checkAddedTitle = addCheckBox("TOP-Überschrift weicht von Quelle ab");
		checkAddedAll = addCheckBox("TOP-Nummer und -Überschrift weichen von Quelle ab");

		checkBoxGroup.add(checkAddedNumber);
		checkBoxGroup.add(checkAddedTitle);
		checkBoxGroup.add(checkAddedAll);

		addEmptySpace();
	}

	private void addWeiMProtCheckboxes() {
		checkAddedTitle = addCheckBox("TOP-Überschrift vom Beabeiter gebildet");
		checkAddedAll = addCheckBox("TOP-Nummer und -Überschrift von Bearbeiter*in gebildet");
		checkMovedTitle = addCheckBox("TOP-Überschrift umplatziert");
		checkMovedAll = addCheckBox("TOP-Nummer und -Überschrift umplatziert");
		addEmptySpace();
		checkHiddenTop = addCheckBox("Schatten-Tagesordnungspunkt");

		checkBoxGroup.add(checkAddedTitle);
		checkBoxGroup.add(checkAddedAll);
		checkBoxGroup.add(checkMovedTitle);
		checkBoxGroup.add(checkMovedAll);
		checkBoxGroup.add(checkHiddenTop);

		addEmptySpace();
	}

	/**
	 * Fügt dem Dialogfenster eine Checkbox für das erstellen von
	 * Schattenunterpunkten hinzu
	 */
	public void addHiddenTopicCheckbox() {
		addEmptySpace();
		checkHiddenTop = addCheckBox("Ist Schattenunterpunkt");
		checkBoxGroup.add(checkHiddenTop);
	}

	/**
	 * Zeigt das Dialogfenster ab und gibt ein Datenhaltungsobjekt zurück, nachdem
	 * es gschlossen wurde
	 */
	public InsertProtocolTopicBean displayDialog() {
		inputBean = new InsertProtocolTopicBean();
		okButton.addActionListener(new InsertProtocolTopicOkButtonListener());

		readyDialog();
		return inputBean;
	}

	// Verhalten für den "Einfügen"-Button des Fensters
	private class InsertProtocolTopicOkButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {

			boolean inputValid = validateInput();

			if (inputValid) {
				extractInput();
				dispose();
			}
		}

		// Überprüft, ob die Benutzereingaben vollständig und gültig sind
		private boolean validateInput() {

			boolean allValid = true;
			String notAllValidMessage = "";

			if (!DialogUtility.isFilledTextfield(topNumField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für die TOP-Nummer eingegeben;\n";
			}
			if (!DialogUtility.isFilledTextfield(normNumField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für die TOP-Nummer (arabische Ziffern/Großbuchstaben) eingegeben;\n";
			} else if (!isValidNormNumber(normNumField)) {
				allValid = false;
				notAllValidMessage += "Wert für TOP-Nummer (arabische Ziffern/Großbuchstaben) ungültig. Keine Zeichen außer den genannten sind erlaubt.";
			}

			if (isHiddenTop() && DialogUtility.isFilledTextfield(titleField)) {
				allValid = false;
				notAllValidMessage += "Für Schatten-Tagesordnungspunkte darf keine Überschrift eingegeben werden";
			}

			if (!isHiddenTop() && !DialogUtility.isFilledTextfield(titleField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für Überschrift eingegeben; \n";
			}

			if (allValid) {
				return true;
			} else {
				displayInputNotAllValidMessage(notAllValidMessage);
				return false;
			}
		}

		// Überprüft ob die maschinenlesbare TOP-Nummer ein gültiges Format
		// besitzt
		private boolean isValidNormNumber(JTextField normNumField) {

			String normNumber = normNumField.getText();
			return normNumber.matches("[A-Z]+|\\d+");
		}

		// Überprüft ob Schattenunterpunkt erstellt werden soll
		private boolean isHiddenTop() {

			if (checkHiddenTop != null && checkHiddenTop.isSelected()) {
				return true;
			} else {
				return false;
			}
		}

		// Überträgt Benutzereingaben in Datenhaltungsobjekt
		private void extractInput() {

			inputBean = new InsertProtocolTopicBean();

			inputBean.elementNum = topNumField.getText();
			inputBean.normNum = normNumField.getText();
			inputBean.title = titleField.getText();
			inputBean.addedTitle = checkAddedTitle.isSelected();
			inputBean.addedAll = checkAddedAll.isSelected();

			// Auswertung abhängig von der aufrufenden Operation
			if (dialogType == 0 || dialogType == 1) {
				inputBean.addedNumber = checkAddedNumber.isSelected();
			} else if (dialogType == 2 || dialogType == 3) {
				inputBean.movedTitle = checkMovedTitle.isSelected();
				inputBean.movedAll = checkMovedAll.isSelected();
			}
			inputBean.ishiddenTopic = isHiddenTop();
			inputBean.isFilled = true;

		}
	}
}
