package de.historischekommission_muenchen.oxydition.protedition.common.operations;

import de.historischekommission_muenchen.oxydition.common.beans.SingleInputBean;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.protedition.common.dialogs.InsertNoteIDReferenceDialog;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einen Verweis auf eine
 * Anmerkung aus der Edition einfügen können. Hierfür müssen sie das xml:id des
 * entsprechenden <note>-Elements heraussuchen und in das Dialogfenster, das die
 * Operation öffnet, eintragen.
 * 
 * 
 * @author Maximilian Schrott
 *
 */

public class InsertNoteIDReference extends OxyditionOperation {

	@Override
	public String getDescription() {
		return "Fügt eine Referenz auf irgendeine Anmerkung aus der Edition an der Stelle des Textcursors ein";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap arg1)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		InsertNoteIDReferenceDialog dlg = new InsertNoteIDReferenceDialog(oxygenFrame,
				"Verknüpfung auf Fußnote einfügen");
		SingleInputBean input = dlg.displayDialog();

		if (input.isFilled) {
			String noteReferenceFragment = constructNoteReferenceXMLFragment(input);
			documentController.insertXMLFragment(noteReferenceFragment, caretPosition);
		}

	}

	private String constructNoteReferenceXMLFragment(SingleInputBean input) {

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder
				.append("<ref " + XMLUtility.TEIXMLNS + " type='Anmerkungsreferenz' target='" + input.inputText + "'>");
		fragmentBuilder.append("Anm.-Ref.:" + input.inputText);
		fragmentBuilder.append("</ref>");
		return fragmentBuilder.toString();
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return null;
	}

}
