package de.historischekommission_muenchen.oxydition.protedition.common.utilities;

import javax.swing.JTextField;

import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.api.schemaaware.SchemaAwareHandlerResult;
import ro.sync.ecss.extensions.api.schemaaware.SchemaAwareHandlerResultInsertConstants;
import ro.sync.ecss.extensions.commons.operations.MoveCaretUtil;

/**
 * Stellt Funktionen für die Durchführung von Protokoll-Operationen zur
 * Verfügung
 * 
 * @author Maximilian Schrott
 *
 */
public class ProtocolUtility {

	/**
	 * Überprüft ob die Eingabe im übergebenen Texteingabefeld eine gültige
	 * Protokollnummer ist und gibt Ergebnis zurück
	 */
	public static boolean isValidProtocolNumberInput(JTextField protocolNumberInput) {
		String protocolNumber = protocolNumberInput.getText();
		return protocolNumber.matches("[0-9]{1,3}[a-z]?");
	}

	/**
	 * Liefert die xml:id des TEI-Knoten des aktuellen Protokolls zurück
	 */
	public static AttrValue getProtocolID(AuthorDocumentController documentController) {

		AuthorElement teiNode = documentController.getAuthorDocumentNode().getRootElement();

		return teiNode.getAttribute("xml:id");

	}

	/**
	 * Liefert das Bandkürzel des aktuell bearbeiten Dokuments zurück
	 */
	public static String getVolumeID(AuthorDocumentController documentController) throws AuthorOperationException {

		AuthorNode[] volumeNodes = documentController.findNodesByXPath("//titleStmt/title[@type='Band']", true, true,
				true);
		// Ist kein Bandkürzel im Dokument hinterlegt dann gib null zurück
		if (volumeNodes.length == 0) {
			return null;
		}
		AuthorNode volumeNode = volumeNodes[0];
		AuthorElement volumeElement = (AuthorElement) volumeNode;
		AttrValue volumeIDAttr = volumeElement.getAttribute("n");
		return volumeIDAttr.getRawValue();

	}

	/**
	 * Verschiebt nach einer Einfügeoperation den Mauscursor an die Stelle der
	 * Einfügung
	 * 
	 * @param insertResult - InsertResult der durchgeführten Operation
	 * @param authorAccess - Author-Access-Element für das bearbeitete Dokument
	 */
	public static void moveCursorToInsertLocation(SchemaAwareHandlerResult insertResult, AuthorAccess authorAccess) {

		Integer headerOffset = (Integer) insertResult
				.getResult(SchemaAwareHandlerResultInsertConstants.RESULT_ID_HANDLE_INSERT_FRAGMENT_OFFSET);

		if (headerOffset != null) {
			int headerOffsetInt = headerOffset.intValue();

			MoveCaretUtil.moveCaretToImposedEditorVariableOffset(authorAccess, headerOffsetInt);
		}
	}

	/**
	 * fügt zu einer Zahl als String führende Nullen hinzu, bis eine bestimmte
	 * Textlänge erreicht ist
	 * 
	 * @param number       - die Ausgangszahl
	 * @param targetLength - die gewünschte Länge der Ergebniszahl mit
	 *                     vorangstellten Nullen
	 * @return die übergebene Zahl mit einer ausreichenden Menge vorangestellter
	 *         Nullen
	 */
	public static String prefixZerosForLength(String number, int targetLength) {

		while (number.length() < targetLength) {
			number = "0" + number;
		}

		return number;
	}

}
