package de.historischekommission_muenchen.oxydition.protedition.weimprot.dialogs;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JTextField;

import de.historischekommission_muenchen.oxydition.common.dialogs.ModularGridBagDialog;
import de.historischekommission_muenchen.oxydition.common.utilities.DialogUtility;
import de.historischekommission_muenchen.oxydition.protedition.common.beans.InsertProtocolHeaderBean;
import de.historischekommission_muenchen.oxydition.protedition.common.utilities.ProtocolUtility;

/**
 * Dialogfenster für die Operation zum Einfügen den Dokumentenkopfes eines
 * WeiMProt-Protokolls. Enthält mehrer Textfelder, die die jeweilige Eingabe auf
 * die Gültigkeit überprüfen.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertProtocolHeaderDialog_WeiMProt extends ModularGridBagDialog {

	/**
	 * 
	 */
	private static final long serialVersionUID = -8365605272232727582L;

	private JTextField protNumField;
	private JTextField titleField;
	private JTextField sourceDateField;
	private JTextField placeField;
	private JTextField normDateField;
	private JTextField sourceBeginField;
	private JTextField normBeginField;
	private JTextField sourceEndField;
	private JTextField normEndField;
	private InsertProtocolHeaderBean inputBean;

	public InsertProtocolHeaderDialog_WeiMProt(JFrame parentFrame) {
		super(parentFrame, "Dokumentenkopf einfügen");

		protNumField = addField("Protokollnummer");
		titleField = addField("Titel des Dokuments");
		titleField.setText("Ministerratssitzung");
		placeField = addField("Sitzungsort");
		sourceDateField = addField("Datum laut Quelle");
		normDateField = addField("Formatiertes Datum (YYYY-MM-DD)");
		sourceBeginField = addField("Beginnzeitpunkt laut Quelle");
		normBeginField = addField("Formatierter Beginnzeitpunkt (hh:mm)");
		sourceEndField = addField("Endzeitpunkt laut Quelle");
		normEndField = addField("Formatierter Endzeitpunkt (hh:mm)");

		insertOperationEndButtonPanel("Einfügen", "Abbrechen");
	}

	/**
	 * Aktiviert das Dialogfenster und gibt nach dem Schließen die Benutzereingaben
	 * zurück
	 */
	public InsertProtocolHeaderBean displayDialog() {

		inputBean = new InsertProtocolHeaderBean();
		okButton.addActionListener(new InsertProtocolHeaderOkButtonListener());

		readyDialog();
		return inputBean;
	}

	// Verhalten beim Drücken des "Einfügen"-Knopfes
	private class InsertProtocolHeaderOkButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent arg0) {

			boolean inputValid = validateInput();

			if (inputValid) {
				extractInput();
				dispose();
			}

		}

		// Überprüft ob Nutzereingaben vollständig und korrekt
		private boolean validateInput() {

			boolean allValid = true;
			String notAllValidMessage = "";

			if (!DialogUtility.isFilledTextfield(protNumField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für die Protokollnummer eingetragen; \n";
			} else if (!ProtocolUtility.isValidProtocolNumberInput(protNumField)) {
				allValid = false;
				notAllValidMessage += "Die eingegebene Protokollnummer hat keine gültige Form; \n";
			}

			if (!DialogUtility.isFilledTextfield(titleField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für Titel des Dokuments eingegeben; \n";
			}

			if (!DialogUtility.isFilledTextfield(placeField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für den Sitzungsort eingegeben; \n";
			}

			if (!DialogUtility.isFilledTextfield(sourceDateField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für Datum laut Quelle eingegeben; \n";
			}

			if (!DialogUtility.isFilledTextfield(normDateField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für formatiertes Datum eingetragen; \n";
			} else if (!DialogUtility.isValidNormalizedDateInput(normDateField)) {
				allValid = false;
				notAllValidMessage += "Das eingetragene formatierte Datum ist ungültig; \n";
			}

			// Eingabe bei Zeitfeldern nur ungültig wenn eines der beiden
			// zusammengehörenden Felder ausgefüllt ist und das andere nicht
			if (!DialogUtility.isFilledTextfield(sourceBeginField) && DialogUtility.isFilledTextfield(normBeginField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für Beginnzeitpunkt laut Quelle eingetragen; \n";
			}

			if (!DialogUtility.isFilledTextfield(normBeginField) && DialogUtility.isFilledTextfield(sourceBeginField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für formatierten Beginnzeitpunkt eingetragen; \n";
			} else if (DialogUtility.isFilledTextfield(sourceBeginField)
					&& !DialogUtility.isValidNormalizedTimeInput(normBeginField)) {
				allValid = false;
				notAllValidMessage += "Der eingetrage formatierte Beginnzeitpunkt ist ungültig; \n";
			}

			if (!DialogUtility.isFilledTextfield(sourceEndField) && DialogUtility.isFilledTextfield(normEndField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für formatiertes Datum eingetragen; \n";
			}
			if (!DialogUtility.isFilledTextfield(normEndField) && DialogUtility.isFilledTextfield(sourceEndField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für Endzeitpunkt laut Quelle eingetragen; \n";
			} else if (DialogUtility.isFilledTextfield(sourceEndField)
					&& !DialogUtility.isValidNormalizedTimeInput(normEndField)) {
				allValid = false;
				notAllValidMessage += "Der eingetragene formatierte Endzeitpunkt ist ungültig; \n";
			}

			if (allValid) {
				return true;
			} else {
				displayInputNotAllValidMessage(notAllValidMessage);
				return false;
			}

		}

		// Überträgt Benutzereingaben in Datenhaltungsobjekt
		private void extractInput() {

			inputBean = new InsertProtocolHeaderBean();

			inputBean.isFilled = true;
			inputBean.protNum = protNumField.getText();
			inputBean.protNumWithLeadingZeros = addLeadingZerosToProtNum(protNumField.getText());
			inputBean.title = titleField.getText();
			inputBean.place = placeField.getText();
			inputBean.sourceDate = sourceDateField.getText();
			inputBean.normDate = normDateField.getText();
			inputBean.sourceBegin = sourceBeginField.getText();
			inputBean.normBegin = normBeginField.getText();
			inputBean.sourceEnd = sourceEndField.getText();
			inputBean.normEnd = normEndField.getText();

		}

		// Ergänzt die Protokollnummer mit führenden Nullen auf drei Stellen
		// (vier Stellen falls die letzte Stelle ein Buchstabe ist)
		private String addLeadingZerosToProtNum(String protNum) {

			if (protNum.length() == 1) {
				return "00" + protNum;
			} else if (protNum.length() == 2) {

				if (Character.isLetter(protNum.charAt(1))) {
					return "00" + protNum;
				} else {
					return "0" + protNum;
				}
			} else if (protNum.length() == 3 && Character.isLetter(protNum.charAt(2))) {
				return "0" + protNum;
			} else {
				return protNum;
			}
		}

	}

}
