package de.historischekommission_muenchen.oxydition.protedition.weimprot.operations;

import javax.swing.JFrame;
import javax.swing.text.BadLocationException;

import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import de.historischekommission_muenchen.oxydition.common.beans.InsertStructureElementBean;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.InsertStructureElement;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.protedition.common.beans.InsertProtocolTopicBean;
import de.historischekommission_muenchen.oxydition.protedition.common.dialogs.InsertProtocolTopicDialog;
import de.historischekommission_muenchen.oxydition.protedition.common.utilities.ProtocolUtility;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen den einen
 * (Schatten-)Tagesordungspunkt in ein WeiMProt-Protokolls einfügen können. Die
 * Operation nimmt Benutzereingaben zu Top-Nummer und Überschrift und
 * Informationen zu editorischen Änderungen entgegen und fügt ein <div>-Element
 * ins Protokoll ein zusammen mit den Titelangaben als <head>.
 *
 * @author Maximilian Schrott
 */

public class InsertProtocolTopic_WeiMProt extends InsertStructureElement {

	@Override
	public String getDescription() {
		return "Ermöglicht es den Benutzer*innen einen Tagesordnungspunkt mit Struktur, Nummer und Titel an der Cursorposition in ein  MinRProt-Protokoll einzufügen."
				+ " Verschiedene Sonderfälle sind berücksichtigt";
	}

	@Override
	protected InsertProtocolTopicBean getInputData(JFrame oxygenFrame, String dialogTitle) {
		InsertProtocolTopicDialog dlg = new InsertProtocolTopicDialog(oxygenFrame,
				InsertProtocolTopicDialog.WEIMPROT_TOP);
		InsertProtocolTopicBean input = dlg.displayDialog();

		return input;

	}

	@Override
	protected String constructTopicXMLFragment(InsertStructureElementBean inputBean, String idPrefix) {

		InsertProtocolTopicBean input = (InsertProtocolTopicBean) inputBean;

		String normNum = normaliseNormNum(input.normNum);

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<div " + XMLUtility.TEIXMLNS + " ");

		if (input.ishiddenTopic) {
			fragmentBuilder.append("type='Schatten-Top' ");
		} else {
			fragmentBuilder.append("type='Top' ");
		}
		fragmentBuilder.append("n='" + normNum + "' ");
		fragmentBuilder.append("xml:id='" + idPrefix + "t" + normNum + "'>");
		fragmentBuilder.append(constructHeadXMLFragment(input));
		fragmentBuilder.append("</div>");

		return fragmentBuilder.toString();

	}

	@Override
	protected String constructHeadXMLFragment(InsertStructureElementBean inputBean) {

		InsertProtocolTopicBean input = (InsertProtocolTopicBean) inputBean;

		StringBuilder headBuilder = new StringBuilder();
		headBuilder.append("<head " + XMLUtility.TEIXMLNS + ">");

		if (input.addedAll) {
			headBuilder.append("<add " + XMLUtility.TEIXMLNS + ">");
		} else if (input.movedAll) {
			headBuilder.append("<reg " + XMLUtility.TEIXMLNS + " >");
		}

		headBuilder.append("<num " + XMLUtility.TEIXMLNS + ">" + input.elementNum + "</num>");

		if (input.addedTitle) {
			headBuilder.append("<add " + XMLUtility.TEIXMLNS + ">");
		} else if (input.movedTitle) {
			headBuilder.append("<reg " + XMLUtility.TEIXMLNS + " >");
		}

		headBuilder.append(input.title);

		if (input.addedAll || input.addedTitle) {
			headBuilder.append("</add>");
		} else if (input.movedTitle || input.movedAll) {
			headBuilder.append("</reg>");
		}

		headBuilder.append("</head>");

		return headBuilder.toString();

	}

	@Override
	protected boolean isCaretPositionValid(int caretPosition, AuthorDocumentController documentController)
			throws BadLocationException {
		AuthorElement surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

		return XMLUtility.elementTypeEquals(surroundNode, "Protokoll");
	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "An dieser Position darf kein Tagesordnungspunkt eingefügt werden.\n"
				+ "Fügen Sie einen Tagesordungspunkt innerhalb des 'Protokoll'-Abschnitts ein,\n"
				+ "aber nicht innerhalb eines anderen Gliederungsabschnitts";

	}

	@Override
	protected String findIdPrefix(int caretPosition, AuthorDocumentController documentController)
			throws BadLocationException {

		AttrValue protIdVal = ProtocolUtility.getProtocolID(documentController);
		return protIdVal.getRawValue();

	}

}
