package de.historischekommission_muenchen.oxydition.protedition.weimprot.operations;

import javax.swing.JOptionPane;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.protedition.common.utilities.ProtocolUtility;
import de.historischekommission_muenchen.oxydition.protedition.weimprot.beans.InsertSourceDocumentHeaderBean;
import de.historischekommission_muenchen.oxydition.protedition.weimprot.dialogs.InsertSourceDocumentHeaderDialog;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorConstants;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.api.schemaaware.SchemaAwareHandlerResult;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen den Dokumentenkopf eines
 * Quelldokuments in WeiMProt einfügen können. Die hierfür nötigen Angaben zu
 * Dokumentennummer und -titel machen die Benutzer*innen in einem Dialogfenster.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertSourceDocumentHeader extends OxyditionOperation {

	@Override
	public String getDescription() {
		return "Öffnet Dialogfenster in denen Nummer und Titel des Quelldokuments abgefragt werden. "
				+ "Mit den eingegebenen Daten wird dann der Dokumentenkopf, die DokumentenId und die "
				+ "Quelldokumentsnummer im teiHeader gebildet.";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		boolean noHead = checkIfDocumentAlreadyHasHead(documentController);

		if (!noHead) {
			JOptionPane.showMessageDialog(oxygenFrame,
					"Es kann nicht mehr als ein Dokumentenkopf pro Protokoll eingefügt werden",
					"Operation nicht durchführbar", JOptionPane.WARNING_MESSAGE);
		}

		InsertSourceDocumentHeaderDialog dlg = new InsertSourceDocumentHeaderDialog(oxygenFrame);

		InsertSourceDocumentHeaderBean input = dlg.displayDialog();

		if (input.isFilled) {

			String xPathLocation = "//div[@type='Quelldokument']";
			String volumeId = ProtocolUtility.getVolumeID(documentController);
			insertSourceDocumentNumberIntoHeader(input, documentController);
			insertSourceDocumentXMLID(volumeId, input.sourceDocNumWithLeadingZeros, documentController);
			String sourceDocHeaderFragment = constructSourceDocumentHeaderFragment(input);
			SchemaAwareHandlerResult insertResult = documentController.insertXMLFragmentSchemaAware(
					sourceDocHeaderFragment, xPathLocation, AuthorConstants.POSITION_INSIDE_FIRST);

			if (insertResult != null) {
				ProtocolUtility.moveCursorToInsertLocation(insertResult, authorAccess);
			}
		}

	}

	private boolean checkIfDocumentAlreadyHasHead(AuthorDocumentController documentController)
			throws AuthorOperationException {

		AuthorNode[] numNodes = documentController.findNodesByXPath("//div[@type='Quelldokument']/head/num", false,
				false, true);

		return numNodes.length == 0;
	}

	private void insertSourceDocumentNumberIntoHeader(InsertSourceDocumentHeaderBean input,
			AuthorDocumentController documentController) throws AuthorOperationException {

		String titlePath = "//titleStmt/title[@type='Quelldokument']";

		String titleSourceDocNumber = "d" + input.sourceDocNumWithLeadingZeros;
		AttrValue titleSourceDocNumberAttr = new AttrValue(titleSourceDocNumber);

		AuthorNode[] sourceDocTitleNodes = documentController.findNodesByXPath(titlePath, false, false, true);
		AuthorElement sourceDocTitleElement = (AuthorElement) sourceDocTitleNodes[0];

		documentController.insertXMLFragment(input.sourceDocNum, sourceDocTitleElement,
				AuthorConstants.POSITION_INSIDE_FIRST);
		documentController.setAttribute("n", titleSourceDocNumberAttr, sourceDocTitleElement);
	}

	private void insertSourceDocumentXMLID(String volumeId, String sourceDocNumWithLeadingZeros,
			AuthorDocumentController documentController) {

		AuthorElement teiNode = documentController.getAuthorDocumentNode().getRootElement();

		String sourceDocId = volumeId + "d" + sourceDocNumWithLeadingZeros;
		AttrValue sourceDocIdAttr = new AttrValue(sourceDocId);

		documentController.setAttribute("xml:id", sourceDocIdAttr, teiNode);
	}

	private String constructSourceDocumentHeaderFragment(InsertSourceDocumentHeaderBean input) {

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<head " + XMLUtility.TEIXMLNS + ">");
		fragmentBuilder.append("<num " + XMLUtility.TEIXMLNS + " n='" + input.sourceDocNumWithLeadingZeros + "'>");
		fragmentBuilder.append(input.sourceDocNum + "</num>");

		String sourceDocTitle = XMLUtility.escapeXML(input.sourceDocTitle);
		fragmentBuilder.append("<title " + XMLUtility.TEIXMLNS + ">" + sourceDocTitle + "</title>");
		fragmentBuilder.append("</head>");

		return fragmentBuilder.toString();
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

}
