package de.historischekommission_muenchen.oxydition.register.adaptions.weimrprot;

import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.register.operations.RegisterInsertPersonQuickReference;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Anpassung der Operation zum Erstellen eines Biogrammrumpf für die Edition
 * WeiMProt. Liest, abweichend von der Standartoperation die Elemente
 * <persName @LesenameW/>, <floruit/>, <birth/>/<placeName/>, <faith/> und
 * <affiliation/> ein erzeugt daraus den Biogrammrumpf.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterInsertQuickReference_WeiMProt extends RegisterInsertPersonQuickReference {

	@Override
	public String getDescription() {
		return "Liest den Inhalt der Datenfelder \"Lesename\", \"Qualifikator\", \"Geburtsdatum\" und \"Konfession\" und \"Parteizugehörigkeit\"  aus und "
				+ "fügt sie in das Feld \"Kurzreferenz\" bzw. \"Biogramm\" ein "
				+ "(der bisherige Inhalt des Feldes wird dabei gelöscht) ";
	}

	@Override
	protected String constructQuickReferenceFragment(AuthorElement personNode) throws BadLocationException {

		String nameText = XMLUtility.getChildText(personNode, "persName", "Lesename");
		String floruittext = XMLUtility.getChildText(personNode, "floruit");
		AuthorElement birthNode = XMLUtility.getFirstChildByLocalName(personNode, "birth");
		String birthPlaceText = XMLUtility.getChildText(birthNode, "placeName");
		String faithText = XMLUtility.getChildText(personNode, "faith");
		String affiliationText = XMLUtility.getChildText(personNode, "affiliation");
		String occupationText = XMLUtility.getChildText(personNode, "occupation");

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append(nameText);

		if (floruittext.length() > 0) {
			fragmentBuilder.append(" (" + floruittext + ")");
		}
		fragmentBuilder.append(", ");
		if (birthPlaceText.length() > 0) {
			fragmentBuilder.append("geb. in " + birthPlaceText + ", ");
		}
		if (faithText.length() > 0) {
			fragmentBuilder.append(faithText + ", ");
		}

		if (affiliationText.length() > 0) {
			fragmentBuilder.append(affiliationText + ", ");
		}

		if (occupationText.length() > 0) {
			fragmentBuilder.append(occupationText + ", ");
		}
		return fragmentBuilder.toString();
	}
}
