package de.historischekommission_muenchen.oxydition.register.dialogs;

import java.awt.Color;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import de.historischekommission_muenchen.oxydition.common.dialogs.ModularGridBagDialog;
import de.historischekommission_muenchen.oxydition.register.beans.NewSourceCollectionBean;
import de.historischekommission_muenchen.oxydition.register.beans.SourceCollectionBean;

/**
 * Dialogfenster für die Operation zum Einfügen neuer Quellenbestandseinträge
 * ins Register der ungedruckten Quellen. Zeigt in einer TextArea an, in welches
 * Archiv der neue Bestand eingefügt werden wird.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterNewSourceStockDialog extends ModularGridBagDialog {

	private static final long serialVersionUID = 2740297363116948888L;

	private NewSourceCollectionBean input;
	private JTextField normNameField;
	private JTextField fullNameField;
	private JTextArea hierarchyArea;

	public RegisterNewSourceStockDialog(JFrame parentFrame, SourceCollectionBean archiveBean) {
		super(parentFrame, "Neuen Quellenbestande hinzufügen");

		normNameField = addField("Kennname (Abkürzung falls vorhanden)");
		fullNameField = addField("Voller Name (nur notwendig wenn nicht gleichzeitig Kennname)");
		addEmptySpace();

		insertHierarchyArea(archiveBean);
		addEmptySpace();
		insertOperationEndButtonPanel("Einfügen", "Abbrechen");

	}

	public NewSourceCollectionBean displayDialog() {
		input = new SourceCollectionBean();
		okButton.addActionListener(new OkButtonListener());
		readyDialog();

		return input;
	}

	private void insertHierarchyArea(SourceCollectionBean archiveBean) {

		String hierarchyText = constructHierarchyText(archiveBean);

		hierarchyArea = addFullWidthTextArea(hierarchyText);

		hierarchyArea.setColumns(20);
		hierarchyArea.setLineWrap(true);
		hierarchyArea.setWrapStyleWord(true);
		hierarchyArea.setOpaque(false);
		hierarchyArea.setMargin(new Insets(6, 6, 6, 6));
		hierarchyArea.setEditable(false);
		hierarchyArea.setDisabledTextColor(new Color(0));

	}

	private String constructHierarchyText(SourceCollectionBean archiveBean) {

		StringBuilder hierarchyBuilder = new StringBuilder();
		hierarchyBuilder.append("Der Bestand wird als Teil des Archivs ");
		hierarchyBuilder.append(archiveBean.normName);

		if (!archiveBean.fullName.isEmpty()) {
			hierarchyBuilder.append(" [" + archiveBean.fullName + "]");
		}

		hierarchyBuilder.append(" eingefügt.");

		return hierarchyBuilder.toString();
	}

	private class OkButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {

			String normNameText = normNameField.getText();
			String fullNameText = fullNameField.getText();
			if (normNameText.isEmpty()) {
				displayInputNotAllValidMessage("Es wurde kein Wert für Kennname eingegeben");
			} else {
				input.normName = normNameText;
				input.fullName = fullNameText;
				input.isFilled = true;

				dispose();
			}
		}

	}
}
