package de.historischekommission_muenchen.oxydition.register.dialogs;

import java.awt.Color;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import de.historischekommission_muenchen.oxydition.common.dialogs.ModularGridBagDialog;
import de.historischekommission_muenchen.oxydition.register.beans.CollectionHierarchyBean;
import de.historischekommission_muenchen.oxydition.register.beans.NewUnpublishedSourceBean;
import de.historischekommission_muenchen.oxydition.register.beans.SourceCollectionBean;

/**
 * Dialogfenster für die Operation zum Einfügen neuer Quelleneinträge in die
 * Registerdatenbank für ungedruckten Quellen. Zeigt in einer Textarea an, in
 * welches Archiv und/oder Bestand die neue Quelle eingetragen werden wird.
 * 
 * @author Maximilian Schrott
 * 
 */
public class RegisterNewUnpublishedSourceDialog extends ModularGridBagDialog {

	private static final long serialVersionUID = 365259964306053919L;

	JTextField titleField;
	JTextField descField;
	JTextArea hierarchyArea;

	NewUnpublishedSourceBean input;

	public RegisterNewUnpublishedSourceDialog(JFrame owner, CollectionHierarchyBean hierarchyBean) {

		super(owner, "Neuen Quelleneintrag einfügen");

		titleField = addField("Quellennummer/-bezeichner");
		addEmptySpace();
		descField = addField("Beschreibender Zusatz (optional)");
		addEmptySpace();
		insertHierarchyArea(hierarchyBean);
		addEmptySpace();
		insertOperationEndButtonPanel("Einfügen", "Abbrechen");

	}

	public NewUnpublishedSourceBean displayDialog() {
		input = new NewUnpublishedSourceBean();
		okButton.addActionListener(new OkButtonListener());
		readyDialog();

		return input;
	}

	private void insertHierarchyArea(CollectionHierarchyBean hierarchyBean) {

		String hierarchyText = constructHierarchyText(hierarchyBean);

		hierarchyArea = addFullWidthTextArea(hierarchyText);

		hierarchyArea.setColumns(20);
		hierarchyArea.setLineWrap(true);
		hierarchyArea.setWrapStyleWord(true);
		hierarchyArea.setOpaque(false);
		hierarchyArea.setMargin(new Insets(6, 6, 6, 6));
		hierarchyArea.setEditable(false);
		hierarchyArea.setDisabledTextColor(new Color(0));

	}

	private String constructHierarchyText(CollectionHierarchyBean hierarchyBean) {

		StringBuilder hierarchyBuilder = new StringBuilder();

		SourceCollectionBean archiveBean = hierarchyBean.archiveBean;
		SourceCollectionBean stockBean = hierarchyBean.stockBean;

		hierarchyBuilder.append("Die Quelle wird als Teil ");

		// Variabler Teil, je nachdem, ob Quelle direkt unterhalb eines
		// Bestandes oder Archives einfügt wird
		if (stockBean != null) {
			hierarchyBuilder.append("des Bestandes " + stockBean.normName);

			if (!stockBean.fullName.isEmpty()) {
				hierarchyBuilder.append(" [" + stockBean.fullName + "]");
			}

			hierarchyBuilder.append(" im Archiv ");
		} else {
			hierarchyBuilder.append("des Archivs ");
		}

		hierarchyBuilder.append(archiveBean.normName);
		if (!archiveBean.fullName.isEmpty()) {
			hierarchyBuilder.append(" [" + archiveBean.fullName + "]");
		}
		hierarchyBuilder.append(" eingefügt.");

		return hierarchyBuilder.toString();
	}

	private class OkButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {

			String titleText = titleField.getText();
			String descText = descField.getText();

			if (titleText.isEmpty()) {
				displayInputNotAllValidMessage("Es wurde kein Wert für 'Quellennummer/-bezeichner' eingegben.");
			} else {
				input.title = titleText;
				input.desc = descText;
				input.isFilled = true;

				dispose();
			}

		}
	}

}
