package de.historischekommission_muenchen.oxydition.register.operations;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import de.historischekommission_muenchen.oxydition.common.utilities.StandardReadRegister;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einem
 * Sachschlagworteintrag einen Verweis auf einen Ortseintrag hinzufügen können.
 * 
 * @author MaximilianSchrott
 *
 */
public class RegisterAddGeoReference extends RegisterAddCrossRegisterReference {

	/*
	 * Operationsargumente einrichten
	 */

	private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {
			new ArgumentDescriptor(ARGUMENT_URL, ArgumentDescriptor.TYPE_STRING, urlArgDescription) };

	@Override
	public String getDescription() {
		return "Fügt dem Sachschlagwort an der Cursorposition einen Verweis auf einen Eintrag im Ortsregister hinzu";
	}

	@Override
	protected ReadRegisterBasic getRegisterEntries(ArgumentsMap args)
			throws XPathExpressionException, ParserConfigurationException, IOException, SAXException {
		String urlVal = (String) args.getArgumentValue(ARGUMENT_URL);
		String nodeVal = "//*[local-name()='listPlace']/*[local-name()='place']";
		String expressionArgVal = "//*[local-name()='placeName']";

		return new StandardReadRegister(urlVal, nodeVal, expressionArgVal, idVal);
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "An dieser Stelle kann kein neues Georeferenz eingefügt werden.\n"
				+ "Stellen Sie sicher, dass sich der Textcursor innerhalb eines Schlagworteintrages befindet";
	}

	@Override
	protected String getFrameTitle() {
		return "Ort für Georeferenz wählen";
	}

	@Override
	protected AuthorElement findEntryNode(AuthorElement surroundNode) {
		AuthorElement parentNode = (AuthorElement) surroundNode.getParent();

		// Surroundnode ist das gesuchte <item>-Element
		if (XMLUtility.elementDisplayNameEquals(surroundNode, "item")) {
			if (XMLUtility.elementIsListOfType(parentNode, "Sachschlagwörter")) {
				return surroundNode;
			}
		}
		// Surroundnode innerhalb eines Kindelements des gesuchten <item>-Elements
		else if (XMLUtility.elementDisplayNameEquals(parentNode, "item")) {
			AuthorElement grandParentNode = (AuthorElement) parentNode.getParent();

			if (XMLUtility.elementIsListOfType(grandParentNode, "Sachschlagwörter")) {
				return parentNode;
			}
		}
		return null;
	}

}
