package de.historischekommission_muenchen.oxydition.register.operations;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.utilities.KeywordReadRegister;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einem Ortseintrag einen
 * Verweis auf einen Sachschlagwortseintrag hinzufügen können.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterAddPlaceToTopicReference extends RegisterAddCrossRegisterReference {

	@Override
	protected String getFrameTitle() {
		return "Schlagwort für Sachschlagwortsreferenz wählen";
	}

	@Override
	protected AuthorElement findEntryNode(AuthorElement surroundNode) {

		AuthorElement parentNode = XMLUtility.getNodeParentElement(surroundNode);
		AuthorElement grandParentNode = XMLUtility.getNodeParentElement(parentNode);

		if (XMLUtility.elementDisplayNameEquals(surroundNode, "place")) {
			if (XMLUtility.elementDisplayNameEquals(parentNode, "listPlace")) {
				return surroundNode;
			}
		} else if (XMLUtility.elementDisplayNameEquals(parentNode, "place")) {
			if (XMLUtility.elementDisplayNameEquals(grandParentNode, "listPlace")) {
				return parentNode;
			}
		} else if (XMLUtility.elementDisplayNameEquals(grandParentNode, "place")) {
			AuthorElement greatGrandParentNode = XMLUtility.getNodeParentElement(grandParentNode);
			if (XMLUtility.elementDisplayNameEquals(greatGrandParentNode, "listPlace")) {
				return grandParentNode;
			}
		}
		return null;
	}

	@Override
	protected ReadRegisterBasic getRegisterEntries(ArgumentsMap args)
			throws XPathExpressionException, ParserConfigurationException, IOException, SAXException {

		String urlArgVal = (String) args.getArgumentValue(ARGUMENT_URL);
		return new KeywordReadRegister(urlArgVal);

	}

	@Override
	public String getDescription() {
		return "Fügt dem Ortseintrag an der Position des Textcursors einen Verweis auf einen Eintrag im Sachschlagwortsregister hinzu";
	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "An dieser Position kann kein Verweis auf die Sachschlagwortsregisterdatenbank eingefügt werden.\n"
				+ "Um Die Operation ausführen zu können, platzieren Sie den Textcursor bitte innerhalb eines Ortseintrags.";
	}

}
