package de.historischekommission_muenchen.oxydition.register.operations;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;

/**
 * Abstarkte Überklasse für Operationen mit denen die Benutzer*innen Einträge
 * aus RDBs löschen können, in denen Querverweise zwischen den Einträgen möglich
 * sind. Die Operation löscht nicht nur den Eintrag an der position des
 * Textcursors, sonder auch alle Querverweise die in dr selben Datei auf diesen
 * gesetzt wurden. Erbende Klassen passen die Operation an das jeweilige
 * RDB-Schema an.
 * 
 * @author Maximilian Schrott
 *
 */
public abstract class RegisterDeleteEntry extends OxyditionOperation {

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		AuthorElement surroundNode;

		try {
			surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

			AuthorElement entryNode = nodeWithinEntry(surroundNode);

			if (entryNode == null) {
				displayInvalidArgumentsMessage();
				return;
			}

			int deleteConfirmation = getDeleteConformation(entryNode, oxygenFrame);

			if (deleteConfirmation == JOptionPane.YES_OPTION) {

				deleteItem(entryNode);
			}

		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		}

	}

	protected int getDeleteConformation(AuthorElement entryNode, JFrame parentFrame) throws BadLocationException {

		String entryName = getEntryName(entryNode);

		String deleteMessage = constructDeleteMessage(entryName);

		int confirmation = JOptionPane.showConfirmDialog(parentFrame, deleteMessage, "Löschen bestätigen",
				JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);

		return confirmation;

	}

	protected String constructDeleteMessage(String entryName) {
		return "Wollen sie den Eintrag \"" + entryName + "\" und alle Querverweise auf ihn wirklich löschen.";
	}

	protected void deleteItem(AuthorElement entryNode) throws AuthorOperationException {
		String deleteID = XMLUtility.getAttributeRawValue(entryNode, "xml:id");
		documentController.deleteNode(entryNode);
		deleteCrossReferences(deleteID);

	}

	protected void deleteCrossReferences(String deleteID) throws AuthorOperationException {

		AuthorNode[] lostReferences = documentController
				.findNodesByXPath("//*[@type='Querverweis' and @target='" + deleteID + "']", false, true, true);

		for (int i = 0; i < lostReferences.length; i++) {
			documentController.deleteNode(lostReferences[i]);
		}

	}

	protected abstract AuthorElement nodeWithinEntry(AuthorElement node);

	protected abstract String getEntryName(AuthorElement entryNode) throws BadLocationException;

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Standartmäßig keine Argumente
		return null;
	}

}
