package de.historischekommission_muenchen.oxydition.register.operations;

import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Oxygen-Author-Operation, mit der die Benutzer*innen einen Personeneintrag und
 * alle Querverweise auf ihn aus der RDB löschen können.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterDeletePerson extends RegisterDeleteEntry {

	@Override
	public String getDescription() {
		return "Löscht, nachdem die Benutzer*innen dies bestätigt haben, das <pers>-Element an der Position des Textcursors, sowie alle Querverweise auf diesen Eintrag";
	}

	@Override
	protected AuthorElement nodeWithinEntry(AuthorElement node) {
		AuthorElement persNode = null;
		String nodeDisplayName = node.getDisplayName();

		// Der übergebene Knoten muss entweder ein <Person>-Element sein oder sich
		// innerhalb eines <Person>-Elements befinden
		if (nodeDisplayName.equalsIgnoreCase("person")) {
			persNode = node;
		} else {
			AuthorElement ancestorNode = XMLUtility.getAncestorElementWithLocalName(node, "person");
			persNode = ancestorNode;
		}

		return persNode;
	}

	@Override
	protected String getEntryName(AuthorElement entryNode) throws BadLocationException {

		AuthorElement nameNode = XMLUtility.getFirstChildByLocalNameAndType(entryNode, "persName", "RegisterName");

		return nameNode.getTextContent();
	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "Kein Personeneintrag zum Löschen gewählt."
				+ "\nBitte setzen sie den Textcursor innerhalb eines Schlagworteintrags und versuchen Sie es erneut.";
	}

}
