package de.historischekommission_muenchen.oxydition.register.operations;

import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.beans.SingleInputBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SingleInputDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.register.utilities.RegisterUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Operation mit der die Benutzer*innen einen neuen Personen-Eintrag in die RDB
 * einfügen können.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterNewPerson extends OxyditionOperation {

	@Override
	public String getDescription() {
		return "Öffnet ein Dialogfenster, in dem die Benutzer*innen einen neuen Personeneintrag für das Register eintragen und einfügen können.";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		AuthorElement surroundNode;
		try {
			surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

			if (!XMLUtility.elementDisplayNameEquals(surroundNode, "listPerson")) {
				displayInvalidCaretPositionMessage();
				return;
			}

			SingleInputDialog dlg = new SingleInputDialog(oxygenFrame, "Neue Person eintragen", "Registername");

			SingleInputBean input = dlg.displayDialog();

			if (input.isFilled) {
				insertNewPerson(input);
			}

		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		}
	}

	private void insertNewPerson(SingleInputBean input) throws AuthorOperationException {
		String newPersonFragment = constructNewPersonFragment(input);
		documentController.insertXMLFragment(newPersonFragment, caretPosition);
	}

	protected String constructNewPersonFragment(SingleInputBean input) {

		String personId = RegisterUtility.generateID(input.inputText, false, 20);
		String personName = XMLUtility.escapeXML(input.inputText);
		StringBuilder fragmentBuilder = new StringBuilder();

		fragmentBuilder.append("<person xmlns='http://www.tei-c.org/ns/1.0' xml:id='" + personId + "'>");
		fragmentBuilder.append("<persName type='Registername'>" + personName + "</persName>");
		fragmentBuilder.append("<persName type='Lesename'/>");
		fragmentBuilder.append("<note type='Qualifikator'/>");
		fragmentBuilder.append("<idno type='gnd'/>");
		fragmentBuilder.append("<birth/>");
		fragmentBuilder.append("<death/>");
		fragmentBuilder.append("<note type='Biogramm'/>");
		fragmentBuilder.append("</person>");

		return fragmentBuilder.toString();
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Operationsargumente benötigt
		return null;
	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "An dieser Stelle kann kein neuer Personeneintrag eingefügt werden.\n"
				+ "Fügen Sie den Eintrag bitte innerhalb des Personenliste ein.\n"
				+ "Achten Sie bitte darauf, dass sich der TextCursor nicht innerhalb eines anderen Personeneintrags befindet";
	}
}