package de.historischekommission_muenchen.oxydition.register.operations;

import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.register.beans.NewSourceCollectionBean;
import de.historischekommission_muenchen.oxydition.register.dialogs.RegisterNewSourceArchiveDialog;
import de.historischekommission_muenchen.oxydition.register.utilities.RegisterUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Operation mit der die Benutzer*innen einen neuen Archiv-Eintrag in die RDB
 * für ungedruckte Quellen einfügen können.
 * 
 * @author Maximilian Schrott
 *
 */

public class RegisterNewSourceArchive extends OxyditionOperation {

	@Override
	public String getDescription() {
		return "Dialog zum Einfügen eines neuen Eintrags für ein Archiv für ungedruckte Quellen";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		// Überprüfen ob sich der Textcursor direkt unterhalb eines <list
		// @type="Archive">-Elements befindet
		try {
			AuthorElement surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

			if (!XMLUtility.elementDisplayNameEquals(surroundNode, "list")
					|| !XMLUtility.elementTypeEquals(surroundNode, "Archive")) {
				displayInvalidCaretPositionMessage();
				return;
			}

			RegisterNewSourceArchiveDialog dlg = new RegisterNewSourceArchiveDialog(oxygenFrame);

			NewSourceCollectionBean input = dlg.displayDialog();

			if (input.isFilled) {
				insertNewSourceArchive(input);

			}

		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		}
	}

	private void insertNewSourceArchive(NewSourceCollectionBean input) throws AuthorOperationException {
		String newSourceArchiveFrament = constructNewSourceArchiveXMLFragment(input);
		documentController.insertXMLFragment(newSourceArchiveFrament, caretPosition);
	}

	private String constructNewSourceArchiveXMLFragment(NewSourceCollectionBean input) {

		String id = RegisterUtility.generateID(input.normName, false, 20);

		String normName = XMLUtility.escapeXML(input.normName);
		String fullName = XMLUtility.escapeXML(input.fullName);

		String fragment = "<item xmlns='http://www.tei-c.org/ns/1.0' xml:id='" + id + "'>";
		fragment += "<name xmlns='http://www.tei-c.org/ns/1.0' type='Kennname'>" + normName + "</name>";

		// Vollname optional
		if (!fullName.isEmpty()) {
			fragment += "<name xmlns='http://www.tei-c.org/ns/1.0' type='Vollname'>" + fullName + "</name>";
		}
		fragment += "</item>";

		return fragment;
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

	@Override
	protected Object getInvalidCaretPositionMessage() {
		return "An dieser Stelle kann kein neuer Archiveintrag eingefügt werden.\n"
				+ "Fügen Sie den Eintrag bitte innerhalb der Archivliste ein.\n"
				+ "Achten Sie bitte darauf, dass sich der TextCursor nicht innerhalb eines anderen Archiveeintrags befindet";
	}

}
